@@ -23,6 +23,7 @@
 #include "FilterFactory.h"
 #include "TextFilter.h"
 #include "FilterUtils.h"
+#include "TextConverter.h"
 #include "FilterWrapper.h"
 
 using std::cout;
@@ -31,6 +32,16 @@ using std::string;
 using std::set;
 using namespace Dijon;
 
+static string convertToUTF8(const char *pData, unsigned int dataLen, const string &charset)
+{
+	TextConverter converter(20);
+
+#ifdef DEBUG
+	cout << "FilterWrapper::filterDocument: filter requested conversion from " << charset << endl;
+#endif
+	return converter.toUTF8(pData, dataLen, charset);
+}
+
 FilterWrapper::FilterWrapper(IndexInterface *pIndex) :
 	m_pIndex(pIndex)
 {
@@ -89,6 +100,9 @@ bool FilterWrapper::filterDocument(const Document &doc, const string &originalTy
 
 	if (pFilter != NULL)
 	{
+		// The filter may have to convert the content to UTF-8 itself
+    		pFilter->set_utf8_converter(convertToUTF8);
+
 		fedFilter = FilterUtils::feedFilter(doc, pFilter);
 	}
 	else
@@ -119,6 +133,7 @@ bool FilterWrapper::filterDocument(const Document &doc, const string &originalTy
 	{
 		string actualType(originalType);
 		bool isNested = false;
+		bool emptyTitle = false;
 
 		if (pFilter->next_document() == false)
 		{
@@ -156,19 +171,25 @@ bool FilterWrapper::filterDocument(const Document &doc, const string &originalTy
 		}
 		else if (filteredDoc.getTitle().empty() == true)
 		{
-			Url urlObj(doc.getLocation());
-
-			// Default to the file name as title
-			filteredDoc.setTitle(urlObj.getFile());
-#ifdef DEBUG
-			cout << "FilterWrapper::filterDocument: set default title " << urlObj.getFile() << endl;
-#endif
+			emptyTitle = true;
 		}
 
 		// Pass it down to another filter ?
 		if ((filteredDoc.getType().length() >= 10) &&
 			(filteredDoc.getType().substr(0, 10) == "text/plain"))
 		{
+			// Do we need to set a default title ?
+			if (emptyTitle == true)
+			{
+				Url urlObj(doc.getLocation());
+
+				// Default to the file name as title
+				filteredDoc.setTitle(urlObj.getFile());
+#ifdef DEBUG
+				cout << "FilterWrapper::filterDocument: set default title " << urlObj.getFile() << endl;
+#endif
+			}
+
 			// No, it's been reduced to plain text
 			filteredDoc.setType(actualType);
 
