@@ -19,6 +19,7 @@
 #include <algorithm>
 #include <fstream>
 #include <iostream>
+#include <cstring>
 
 #include "config.h"
 #include "Document.h"
@@ -270,7 +271,8 @@ bool PluginWebEngine::runQuery(QueryProperties& queryProps,
 		if (m_properties.m_scrolling == SearchPluginProperties::PER_INDEX)
 		{
 			paramIter = m_properties.m_parameters.find(SearchPluginProperties::COUNT_PARAM);
-			if (paramIter != m_properties.m_parameters.end())
+			if ((paramIter != m_properties.m_parameters.end()) &&
+				(paramIter->second.empty() == false))
 			{
 				// Number of results requested
 				pageQuery += "&";
@@ -281,7 +283,8 @@ bool PluginWebEngine::runQuery(QueryProperties& queryProps,
 			}
 
 			paramIter = m_properties.m_parameters.find(SearchPluginProperties::START_INDEX_PARAM);
-			if (paramIter != m_properties.m_parameters.end())
+			if ((paramIter != m_properties.m_parameters.end()) &&
+				(paramIter->second.empty() == false))
 			{
 				// The offset of the first result (typically 1 or 0)
 				pageQuery += "&";
@@ -294,7 +297,8 @@ bool PluginWebEngine::runQuery(QueryProperties& queryProps,
 		else
 		{
 			paramIter = m_properties.m_parameters.find(SearchPluginProperties::START_PAGE_PARAM);
-			if (paramIter != m_properties.m_parameters.end())
+			if ((paramIter != m_properties.m_parameters.end()) &&
+				(paramIter->second.empty() == false))
 			{
 				// The offset of the page
 				pageQuery += "&";
