@@ -27,6 +27,7 @@
 
 #include "Timer.h"
 #include "AbstractGenerator.h"
+#include "CJKVTokenizer.h"
 
 using std::cout;
 using std::cerr;
@@ -35,6 +36,7 @@ using std::string;
 using std::vector;
 using std::map;
 using std::find;
+using namespace Dijon;
 
 unsigned int AbstractGenerator::m_maxSeedTerms = 5;
 unsigned int AbstractGenerator::m_minTermPositions = 10;
@@ -64,12 +66,14 @@ AbstractGenerator::~AbstractGenerator()
 string AbstractGenerator::generateAbstract(Xapian::docid docId,
 	const vector<string> &seedTerms)
 {
+	CJKVTokenizer tokenizer;
 	map<Xapian::termpos, PositionWindow> abstractWindows;
 	map<Xapian::termpos, string> wordsBuffer;
 	string summary;
 	Xapian::termpos bestPosition = 0, startPosition = 0;
 	unsigned int seedTermsCount = 0, bestWeight = 0;
 	bool topTerm = true;
+	bool wasCJKV = false;
 
 	if (m_pIndex == NULL)
 	{
@@ -188,6 +192,12 @@ string AbstractGenerator::generateAbstract(Xapian::docid docId,
 			{
 				continue;
 			}
+			// Skip multi-character CJKV terms
+			if ((tokenizer.has_cjkv(termName) == true) &&
+				(termName.length() > 4))
+			{
+				continue;
+			}
 
 			for (Xapian::PositionIterator positionIter = m_pIndex->positionlist_begin(docId, termName);
 				positionIter != m_pIndex->positionlist_end(docId, termName); ++positionIter)
@@ -231,6 +241,13 @@ string AbstractGenerator::generateAbstract(Xapian::docid docId,
 			continue;
 		}
 
+		bool isCJKV = tokenizer.has_cjkv(wordIter->second);
+		if ((summary.empty() == false) &&
+			((wasCJKV != isCJKV) || (isCJKV == false)))
+		{
+			summary += " ";
+		}
+
 		// Is this a seed term ?
 		if (find(seedTerms.begin(), seedTerms.end(), wordIter->second) != seedTerms.end())
 		{
@@ -242,8 +259,8 @@ string AbstractGenerator::generateAbstract(Xapian::docid docId,
 		{
 			summary += pEscToken;
 		}
-		summary += " ";
 
+		wasCJKV = isCJKV;
 		g_free(pEscToken);
 	}
 #ifdef DEBUG
