@@ -21,6 +21,7 @@
 #include <unistd.h>
 #include <time.h>
 #include <string>
+#include <cstring>
 #include <vector>
 #include <iostream>
 #include <algorithm>
@@ -31,6 +32,7 @@
 #include "TimeConverter.h"
 #include "Timer.h"
 #include "Url.h"
+#include "CJKVTokenizer.h"
 #include "XapianDatabaseFactory.h"
 #include "AbstractGenerator.h"
 #include "XapianEngine.h"
@@ -151,6 +153,176 @@ class PrefixDecider : public Xapian::ExpandDecider
 
 };
 
+class QueryModifier : public Dijon::CJKVTokenizer::TokensHandler
+{
+	public:
+		typedef enum { NONE = 0, BRACKETS } CJKVWrap;
+
+		QueryModifier(const string &query, unsigned int nGramSize) :
+			m_query(query),
+			m_pos(0),
+			m_wrap(BRACKETS),
+			m_nGramCount(0),
+			m_nGramSize(nGramSize)
+		{
+		}
+
+		virtual ~QueryModifier()
+		{
+		}
+
+		virtual bool handle_token(const string &tok, bool is_cjkv)
+		{
+			if (tok.empty() == true)
+			{
+				return false;
+			}
+#ifdef DEBUG
+			cout << "QueryModifier::handle_token: " << tok << endl;
+#endif
+
+			// Where is this token in the original query ?
+			string::size_type tokPos = m_query.find(tok, m_pos);
+
+			// Is this CJKV ?
+			if (is_cjkv == false)
+			{
+				char lastChar = tok[tok.length() - 1];
+
+				if (tokPos == string::npos)
+				{
+					// This should have been found
+					return false;
+				}
+
+				if (m_nGramCount > 0)
+				{
+					wrapClose();
+
+					m_nGramCount = 0;
+					m_pos = tokPos;
+				}
+
+				m_currentFilter.clear();
+				if (lastChar == '"')
+				{
+					// It's a quoted string
+					m_wrap = NONE;
+				}
+				else if (lastChar == ':')
+				{
+					// It's a filter
+					m_wrap = NONE;
+					m_currentFilter = tok;
+#ifdef DEBUG
+					cout << "QueryModifier::handle_token: in filter" << endl;
+#endif
+				}
+				else
+				{
+#ifdef DEBUG
+					cout << "QueryModifier::handle_token: in brackets" << endl;
+#endif
+					m_wrap = BRACKETS;
+				}
+
+				// Return right away
+				return true;
+			}
+
+			// First n-gram ?
+#ifdef DEBUG
+			cout << "QueryModifier::handle_token: n-gram " << m_nGramCount << endl;
+#endif
+			if (m_nGramCount == 0)
+			{
+				if (tokPos == string::npos)
+				{
+					// That's definitely not right
+					return false;
+				}
+
+				// Append non-CJKV text that precedes and start wrapping CJKV tokens
+				if (tokPos > m_pos)
+				{
+					m_modifiedQuery += " " + m_query.substr(m_pos, tokPos - m_pos);
+				}
+				m_pos += tok.length();
+				switch (m_wrap)
+				{
+					case BRACKETS:
+						m_modifiedQuery += " (";
+						break;
+					case NONE:
+					default:
+						break;
+				}
+			}
+			else
+			{
+				m_modifiedQuery += " ";
+				if (m_currentFilter.empty() == false)
+				{
+					m_modifiedQuery += m_currentFilter;
+				}
+			}
+			m_modifiedQuery += tok;
+#ifdef DEBUG
+			cout << "QueryModifier::handle_token: " << m_modifiedQuery << endl;
+#endif
+
+			++m_nGramCount;
+			if ((m_nGramCount % m_nGramSize == 0) &&
+				(tokPos != string::npos))
+			{
+				m_pos = tokPos + tok.length();
+			}
+
+			return true;
+		}
+
+		string get_modified_query(void)
+		{
+#ifdef DEBUG
+			cout << "QueryModifier::get_modified_query: " << m_pos << "/" << m_query.length() << endl;
+#endif
+			if (m_pos + 1 < m_query.length())
+			{
+				m_modifiedQuery += " " + m_query.substr(m_pos + 1);
+			}
+			wrapClose();
+#ifdef DEBUG
+			cout << "QueryModifier::get_modified_query: " << m_modifiedQuery << endl;
+#endif
+
+			return m_modifiedQuery;
+		}
+
+	protected:
+		string m_query;
+		string m_modifiedQuery;
+		string::size_type m_pos;
+		CJKVWrap m_wrap;
+		string m_currentFilter;
+		unsigned int m_nGramCount;
+		unsigned int m_nGramSize;
+
+		void wrapClose(void)
+		{
+			// Finish wrapping CJKV tokens
+			switch (m_wrap)
+			{
+				case BRACKETS:
+					m_modifiedQuery += ')';
+					break;
+				case NONE:
+				default:
+					break;
+			}
+		}
+
+};
+
 XapianEngine::XapianEngine(const string &database) :
 	SearchEngineInterface()
 {
@@ -175,10 +347,28 @@ Xapian::Query XapianEngine::parseQuery(Xapian::Database *pIndex, const QueryProp
 {
 	Xapian::QueryParser parser;
 	Xapian::Stem stemmer;
+	CJKVTokenizer tokenizer;
 	string freeQuery(StringManip::replaceSubString(queryProps.getFreeQuery(), "\n", " "));
 	unsigned int minDay, minMonth, minYear = 0;
 	unsigned int maxDay, maxMonth, maxYear = 0;
 
+	if (tokenizer.has_cjkv(freeQuery) == true)
+	{
+		QueryModifier handler(freeQuery, tokenizer.get_ngram_size());
+
+		tokenizer.tokenize(freeQuery, handler, true);
+
+		string cjkvQuery(handler.get_modified_query());
+#ifdef DEBUG
+		cout << "XapianEngine::parseQuery: CJKV query is " << cjkvQuery << endl;
+#endif
+
+		// Do as if the user had given this as input
+		freeQuery = cjkvQuery;
+		// We can disable stemming and spelling correction
+		minimal = true;
+	}
+
 	if (pIndex != NULL)
 	{
 		// The database is required for wildcards and spelling
@@ -230,6 +420,7 @@ Xapian::Query XapianEngine::parseQuery(Xapian::Database *pIndex, const QueryProp
 	parser.add_boolean_prefix("type", "T");
 	parser.add_boolean_prefix("class", "XCLASS:");
 	parser.add_boolean_prefix("label", "XLABEL:");
+	parser.add_boolean_prefix("tokens", "XTOK:");
 
 	// Any limit on what documents should be searched ?
 	if (limitQuery.empty() == false)
@@ -368,24 +559,24 @@ Xapian::Query XapianEngine::parseQuery(Xapian::Database *pIndex, const QueryProp
 #endif
 	}
 	Xapian::Query parsedQuery = parser.parse_query(freeQuery, flags);
-	if (minimal == true)
-	{
-		return parsedQuery;
-	}
 #ifdef DEBUG
 	cout << "XapianEngine::parseQuery: " << parsedQuery.get_description() << endl;
 #endif
 
-#if ENABLE_XAPIAN_SPELLING_CORRECTION>0
-	// Any correction ?
-	correctedFreeQuery = parser.get_corrected_query_string();
-#ifdef DEBUG
-	if (correctedFreeQuery.empty() == false)
+	if (minimal == false)
 	{
-		cout << "XapianEngine::parseQuery: corrected spelling to: " << correctedFreeQuery << endl;
+
+#if ENABLE_XAPIAN_SPELLING_CORRECTION>0
+		// Any correction ?
+		correctedFreeQuery = parser.get_corrected_query_string();
+#ifdef DEBUG
+		if (correctedFreeQuery.empty() == false)
+		{
+			cout << "XapianEngine::parseQuery: corrected spelling to: " << correctedFreeQuery << endl;
+		}
+#endif
+#endif
 	}
-#endif
-#endif
 
 	return parsedQuery;
 }
@@ -463,6 +654,33 @@ bool XapianEngine::queryDatabase(Xapian::Database *pIndex, Xapian::Query &query,
 						cout << "XapianEngine::queryDatabase: matched term " << *termIter << endl;
 #endif
 					}
+					else if (firstChar == 'Z')
+					{
+						string stemmed((*termIter).substr(1));
+						string::size_type stemmedLen = stemmed.length();
+
+						// Which of this document's terms stem to this ?
+						Xapian::TermIterator docTermIter = pIndex->termlist_begin(docId);
+						if (docTermIter != pIndex->termlist_end(docId))
+						{
+							for (docTermIter.skip_to(stemmed);
+								docTermIter != pIndex->termlist_end(docId); ++docTermIter)
+							{
+								// Is this a potential unstem ?
+								if (strncasecmp((*docTermIter).c_str(), stemmed.c_str(), stemmedLen) != 0)
+								{
+									// No, no point looking at the next terms
+									break;
+								}
+#ifdef DEBUG
+								cout << "XapianEngine::queryDatabase: matched unstem " << *docTermIter << endl;
+#endif
+
+								// FIXME: check this term stems to stemmed !
+								seedTerms.push_back(*docTermIter); 
+							}
+						}
+					}
 				}
 
 				DocumentInfo thisResult;
