@@ -29,6 +29,7 @@
 #include <fstream>
 #include <algorithm>
 #include <utility>
+#include <cstring>
 #include <xapian.h>
 
 #include "Languages.h"
@@ -39,6 +40,8 @@
 #include "XapianDatabaseFactory.h"
 #include "XapianIndex.h"
 
+#define MAGIC_TERM "X-MetaSE-Doc"
+
 using std::cout;
 using std::cerr;
 using std::endl;
@@ -85,7 +88,106 @@ static string getVersionFromFile(const string &databaseName)
 	return version;
 }
 
-const string XapianIndex::MAGIC_TERM = "X-MetaSE-Doc";
+class TokensIndexer : public Dijon::CJKVTokenizer::TokensHandler
+{
+	public:
+		TokensIndexer(Xapian::Stem *pStemmer, Xapian::Document &doc,
+			const Xapian::WritableDatabase &db,
+			const string &prefix, unsigned int nGramSize,
+			bool &doSpelling, Xapian::termcount &termPos) :
+			Dijon::CJKVTokenizer::TokensHandler(),
+			m_pStemmer(pStemmer),
+			m_doc(doc),
+			m_db(db),
+			m_prefix(prefix),
+			m_nGramSize(nGramSize),
+			m_nGramCount(0),
+			m_doSpelling(doSpelling),
+			m_termPos(termPos)
+		{
+		}
+
+		virtual ~TokensIndexer()
+		{
+		}
+
+		virtual bool handle_token(const string &tok, bool is_cjkv)
+		{
+			bool addSpelling = false;
+
+			if (tok.empty() == true)
+			{
+				return false;
+			}
+
+			// Lower case the term and trim spaces
+			string term(StringManip::toLowerCase(tok));
+			StringManip::trimSpaces(term);
+
+			if (term.empty() == true)
+			{
+				return true;
+			}
+
+			m_doc.add_posting(m_prefix + XapianDatabase::limitTermLength(term), m_termPos);
+
+			// Is this CJKV ?
+			if (is_cjkv == false)
+			{
+				// Don't stem if the term starts with a digit
+				if ((m_pStemmer != NULL) &&
+					(isdigit((int)term[0]) == 0))
+				{
+					string stemmedTerm((*m_pStemmer)(term));
+
+					m_doc.add_term("Z" + XapianDatabase::limitTermLength(stemmedTerm));
+				}
+
+				addSpelling = m_doSpelling;
+				++m_termPos;
+				m_nGramCount = 0;
+			}
+			else
+			{
+				if (m_nGramCount % m_nGramSize == 0)
+				{
+					++m_termPos;
+				}
+				else if ((m_nGramCount + 1) % m_nGramSize == 0)
+				{
+					addSpelling = m_doSpelling;
+				}
+				++m_nGramCount;
+			}
+
+			if (addSpelling == true)
+			{
+				try
+				{
+					m_db.add_spelling(XapianDatabase::limitTermLength(term));
+				}
+				catch (const Xapian::UnimplementedError &error)
+				{
+					cerr << "Couldn't index with spelling correction: " << error.get_type() << ": " << error.get_msg() << endl;
+
+					m_doSpelling = false;
+				}
+			}
+
+			return true;
+		}
+
+	protected:
+		Xapian::Stem *m_pStemmer;
+		Xapian::Document &m_doc;
+		const Xapian::WritableDatabase &m_db;
+		string m_prefix;
+		unsigned int m_nGramSize;
+		unsigned int m_nGramCount;
+		bool &m_doSpelling;
+		Xapian::termcount &m_termPos;
+
+};
 
 XapianIndex::XapianIndex(const string &indexName) :
 	IndexInterface(),
@@ -182,10 +284,11 @@ bool XapianIndex::listDocumentsWithTerm(const string &term, set<unsigned int> &d
 }
 
 void XapianIndex::addPostingsToDocument(const Xapian::Utf8Iterator &itor, Xapian::Document &doc,
-	const Xapian::WritableDatabase &db, const string &prefix, bool noStemming)
+	const Xapian::WritableDatabase &db, const string &prefix, bool noStemming, bool &doSpelling,
+	Xapian::termcount &termPos) const
 {
 	Xapian::Stem *pStemmer = NULL;
-	Xapian::TermGenerator generator;
+	bool isCJKV = false;
 
 	// Do we know what language to use for stemming ?
 	if ((noStemming == false) &&
@@ -199,39 +302,62 @@ void XapianIndex::addPostingsToDocument(const Xapian::Utf8Iterator &itor, Xapian
 		{
 			cerr << "Couldn't create stemmer: " << error.get_type() << ": " << error.get_msg() << endl;
 		}
+	}
 
+	const char *pRawData = itor.raw();
+	if (pRawData != NULL)
+	{
+		Dijon::CJKVTokenizer tokenizer;
+		string text(pRawData);
+
+		if (tokenizer.has_cjkv(text) == true)
+		{
+			// Use overload
+			addPostingsToDocument(tokenizer, pStemmer, text, doc, db,
+				prefix, doSpelling, termPos);
+			isCJKV = true;
+		}
+	}
+
+	if (isCJKV == false)
+	{
+		Xapian::TermGenerator generator;
+
+		// Set the stemmer
 		if (pStemmer != NULL)
 		{
 			generator.set_stemmer(*pStemmer);
 		}
-	}
 
-	try
-	{
-		// Older Xapian backends don't support spelling correction
-		if (m_doSpelling == true)
+		generator.set_termpos(termPos);
+		try
 		{
-			// The database is required for the spelling dictionary
-			generator.set_flags(Xapian::TermGenerator::FLAG_SPELLING);
-			generator.set_database(db);
-		}
-		generator.set_document(doc);
-		generator.index_text(itor, 1, prefix);
-	}
-	catch (const Xapian::UnimplementedError &error)
-	{
-		cerr << "Couldn't index with spelling correction: " << error.get_type() << ": " << error.get_msg() << endl;
-
-		if (m_doSpelling == true)
-		{
-			m_doSpelling = false;
-
-			// Try again without spelling correction
-			// Let the caller catch the exception
-			generator.set_flags(Xapian::TermGenerator::FLAG_SPELLING, Xapian::TermGenerator::FLAG_SPELLING);
+			// Older Xapian backends don't support spelling correction
+			if (doSpelling == true)
+			{
+				// The database is required for the spelling dictionary
+				generator.set_flags(Xapian::TermGenerator::FLAG_SPELLING);
+				generator.set_database(db);
+			}
 			generator.set_document(doc);
 			generator.index_text(itor, 1, prefix);
 		}
+		catch (const Xapian::UnimplementedError &error)
+		{
+			cerr << "Couldn't index with spelling correction: " << error.get_type() << ": " << error.get_msg() << endl;
+
+			if (doSpelling == true)
+			{
+				doSpelling = false;
+
+				// Try again without spelling correction
+				// Let the caller catch the exception
+				generator.set_flags(Xapian::TermGenerator::FLAG_SPELLING, Xapian::TermGenerator::FLAG_SPELLING);
+				generator.set_document(doc);
+				generator.index_text(itor, 1, prefix);
+			}
+		}
+		termPos = generator.get_termpos();
 	}
 
 	if (pStemmer != NULL)
@@ -240,38 +366,61 @@ void XapianIndex::addPostingsToDocument(const Xapian::Utf8Iterator &itor, Xapian
 	}
 }
 
-void XapianIndex::removePostingsFromDocument(const Xapian::Utf8Iterator &itor, Xapian::Document &doc,
-	const Xapian::WritableDatabase &db, const string &prefix, const string &language, bool noStemming) const
+void XapianIndex::addPostingsToDocument(Dijon::CJKVTokenizer &tokenizer, Xapian::Stem *pStemmer,
+	const string &text, Xapian::Document &doc, const Xapian::WritableDatabase &db,
+	const string &prefix, bool &doSpelling, Xapian::termcount &termPos) const
 {
-	Xapian::Document termsDoc;
-	Xapian::TermGenerator generator;
-	Xapian::Stem *pStemmer = NULL;
-	string stemPrefix("Z");
-	string term;
+	TokensIndexer handler(pStemmer, doc, db, prefix, tokenizer.get_ngram_size(),
+		doSpelling, termPos);
 
-	// Do we know what language to use for stemming ?
-	if ((noStemming == false) &&
-		(language.empty() == false))
+	// Get the terms
+	tokenizer.tokenize(text, handler);
+
+#ifdef DEBUG
+	cout << "XapianIndex::addPostingsToDocument: CJKV terms to position " << termPos << endl;
+#endif
+
+	// This will help identify which documents were processed here
+	doc.add_term("XTOK:CJKV");
+}
+
+void XapianIndex::addLabelsToDocument(Xapian::Document &doc, const set<string> &labels,
+	bool skipInternals)
+{
+	if (labels.empty() == true)
 	{
-		try
-		{
-			pStemmer = new Xapian::Stem(StringManip::toLowerCase(m_stemLanguage));
-		}
-		catch (const Xapian::Error &error)
-		{
-			cerr << "Couldn't create stemmer: " << error.get_type() << ": " << error.get_msg() << endl;
-		}
-
-		if (pStemmer != NULL)
-		{
-			generator.set_stemmer(*pStemmer);
-		}
+		return;
 	}
 
-	// This temporary document enables to get to the same terms
-	// that were added at indexing time
-	generator.set_document(termsDoc);
-	generator.index_text(itor, 1, prefix);
+	for (set<string>::const_iterator labelIter = labels.begin(); labelIter != labels.end();
+		++labelIter)
+	{
+		string labelName(*labelIter);
+
+		// Prevent from setting internal labels ?
+		if ((labelName.empty() == true) ||
+			((skipInternals == true) && (labelName.substr(0, 2) == "X-")))
+		{
+			continue;
+		}
+
+#ifdef DEBUG
+		cout << "XapianIndex::addLabelsToDocument: label \"" << labelName << "\"" << endl;
+#endif
+		doc.add_term(string("XLABEL:") + XapianDatabase::limitTermLength(Url::escapeUrl(labelName)));
+	}
+}
+
+void XapianIndex::removePostingsFromDocument(const Xapian::Utf8Iterator &itor, Xapian::Document &doc,
+	const Xapian::WritableDatabase &db, const string &prefix, const string &language,
+	bool noStemming, bool &doSpelling) const
+{
+	Xapian::Document termsDoc;
+	Xapian::termcount termPos = 0;
+	bool addDoSpelling = false;
+
+	// Get the terms, without populating the spelling database
+	addPostingsToDocument(itor, termsDoc, db, prefix, noStemming, addDoSpelling, termPos);
 
 	// Get the terms and remove the first posting for each
 	for (Xapian::TermIterator termListIter = termsDoc.termlist_begin();
@@ -326,13 +475,27 @@ void XapianIndex::removePostingsFromDocument(const Xapian::Utf8Iterator &itor, X
 			try
 			{
 				doc.remove_term(*termListIter);
+			}
+			catch (const Xapian::Error &error)
+			{
+#ifdef DEBUG
+				cout << "XapianIndex::removePostingsFromDocument: " << error.get_msg() << endl;
+#endif
+			}
 
+			try
+			{
 				// Decrease this term's frequency in the spelling dictionary
-				if (m_doSpelling == true)
+				if (doSpelling == true)
 				{
 					db.remove_spelling(*termListIter);
 				}
 			}
+			catch (const Xapian::UnimplementedError &error)
+			{
+				cerr << "Couldn't remove spelling correction: " << error.get_type() << ": " << error.get_msg() << endl;
+				doSpelling = false;
+			}
 			catch (const Xapian::Error &error)
 			{
 #ifdef DEBUG
@@ -367,15 +530,10 @@ void XapianIndex::removePostingsFromDocument(const Xapian::Utf8Iterator &itor, X
 			}
 		}
 	}
-
-	if (pStemmer != NULL)
-	{
-		delete pStemmer;
-	}
 }
 
 void XapianIndex::addCommonTerms(const DocumentInfo &info, Xapian::Document &doc,
-	const Xapian::WritableDatabase &db)
+	const Xapian::WritableDatabase &db, Xapian::termcount &termPos)
 {
 	string title(info.getTitle());
 	string location(info.getLocation());
@@ -388,8 +546,13 @@ void XapianIndex::addCommonTerms(const DocumentInfo &info, Xapian::Document &doc
 	// Index the title with and without prefix S
 	if (title.empty() == false)
 	{
-		addPostingsToDocument(Xapian::Utf8Iterator(title), doc, db, "S", true);
-		addPostingsToDocument(Xapian::Utf8Iterator(title), doc, db, "", false);
+		addPostingsToDocument(Xapian::Utf8Iterator(title), doc, db, "S",
+			true, m_doSpelling, termPos);
+		addPostingsToDocument(Xapian::Utf8Iterator(title), doc, db, "",
+			false, m_doSpelling, termPos);
+
+		// Make room between common terms and what follows
+		termPos += 100;
 	}
 
 	// Index the full URL with prefix U
@@ -509,8 +672,10 @@ void XapianIndex::removeCommonTerms(Xapian::Document &doc, const Xapian::Writabl
 	string title(docInfo.getTitle());
 	if (title.empty() == false)
 	{
-		removePostingsFromDocument(Xapian::Utf8Iterator(title), doc, db, "S", language, true);
-		removePostingsFromDocument(Xapian::Utf8Iterator(title), doc, db, "", language, false);
+		removePostingsFromDocument(Xapian::Utf8Iterator(title), doc, db, "S",
+			language, true, m_doSpelling);
+		removePostingsFromDocument(Xapian::Utf8Iterator(title), doc, db, "",
+			language, false, m_doSpelling);
 	}
 
 	// Location 
@@ -609,21 +774,34 @@ void XapianIndex::removeCommonTerms(Xapian::Document &doc, const Xapian::Writabl
 	}
 }
 
-string XapianIndex::scanDocument(const char *pData, unsigned int dataLength,
-	DocumentInfo &info)
+string XapianIndex::scanDocument(const string &suggestedLanguage,
+	const char *pData, unsigned int dataLength)
 {
+	LanguageDetector lang;
 	vector<string> candidates;
 	string language;
+	bool scannedDocument = false;
 
-	// Try to determine the document's language
-	LanguageDetector lang;
-	lang.guessLanguage(pData, max(dataLength, (unsigned int)2048), candidates);
+	if (suggestedLanguage.empty() == false)
+	{
+		// See first if this is suitable
+		candidates.push_back(suggestedLanguage);
+	}
+	else
+	{
+		// Try to determine the document's language right away
+		lang.guessLanguage(pData, max(dataLength, (unsigned int)2048), candidates);
+
+		scannedDocument = true;
+	}
 
 	// See which of these languages is suitable for stemming
-	for (vector<string>::iterator langIter = candidates.begin(); langIter != candidates.end(); ++langIter)
+	vector<string>::iterator langIter = candidates.begin();
+	while (langIter != candidates.end())
 	{
 		if (*langIter == "unknown")
 		{
+			++langIter;
 			continue;
 		}
 
@@ -633,7 +811,21 @@ string XapianIndex::scanDocument(const char *pData, unsigned int dataLength,
 		}
 		catch (const Xapian::Error &error)
 		{
-			cerr << "Couldn't create stemmer: " << error.get_type() << ": " << error.get_msg() << endl;
+			cerr << "Invalid language: " << error.get_type() << ": " << error.get_msg() << endl;
+
+			if (scannedDocument == false)
+			{
+				// The suggested language is not suitable
+				candidates.clear();
+				lang.guessLanguage(pData, max(dataLength, (unsigned int)2048), candidates);
+
+				langIter = candidates.begin();
+				scannedDocument = true;
+			}
+			else
+			{
+				++langIter;
+			}
 			continue;
 		}
 
@@ -644,9 +836,6 @@ string XapianIndex::scanDocument(const char *pData, unsigned int dataLength,
 	cout << "XapianIndex::scanDocument: language " << language << endl;
 #endif
 
-	// Update the document's properties
-	info.setLanguage(language);
-
 	return language;
 }
 
@@ -728,12 +917,12 @@ bool XapianIndex::isGood(void) const
 	return m_goodIndex;
 }
 
-/// Gets the version number.
-string XapianIndex::getVersion(void) const
+/// Gets metadata.
+string XapianIndex::getMetadata(const string &name) const
 {
-	string version("0.00");
-
 #if ENABLE_XAPIAN_DB_METADATA>0
+	string metadataValue;
+
 	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
 	if (pDatabase == NULL)
 	{
@@ -748,41 +937,32 @@ string XapianIndex::getVersion(void) const
 		{
 			// If this index type doesn't support metadata, no exception will be thrown
 			// We will just get an empty string
-			version = pIndex->get_metadata("version");
-			if (version.empty() == true)
-			{
-				// Is there a pre-0.80 version file ?
-				version = getVersionFromFile(m_databaseName);
-				if (version.empty() == true)
-				{
-					version = "0.00";
-				}
-			}
+			metadataValue = pIndex->get_metadata(name);
 		}
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "Couldn't get database version: " << error.get_type() << ": " << error.get_msg() << endl;
+		cerr << "Couldn't get metadata: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (...)
 	{
-		cerr << "Couldn't get database version, unknown exception occured" << endl;
+		cerr << "Couldn't get metadata, unknown exception occured" << endl;
 	}
 	pDatabase->unlock();
-#else
-	version = getVersionFromFile(m_databaseName);
-#endif
 
-	return version;
+	return metadataValue;
+#else
+	return "";
+#endif
 }
 
-/// Sets the version number.
-bool XapianIndex::setVersion(const string &version) const
+/// Sets metadata.
+bool XapianIndex::setMetadata(const string &name, const string &value) const
 {
-	bool setVer = false;
-
 #if ENABLE_XAPIAN_DB_METADATA>0
-	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
+	bool setMetadata = false;
+
+	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false);
 	if (pDatabase == NULL)
 	{
 		cerr << "Bad index " << m_databaseName << endl;
@@ -794,44 +974,28 @@ bool XapianIndex::setVersion(const string &version) const
 		Xapian::WritableDatabase *pIndex = pDatabase->writeLock();
 		if (pIndex != NULL)
 		{
-			pIndex->set_metadata("version", version);
-			setVer = true;
+			pIndex->set_metadata(name, value);
+			setMetadata = true;
 		}
 	}
 	catch (const Xapian::UnimplementedError &error)
 	{
-		cerr << "Couldn't set database version, no support for metadata: " << error.get_type() << ": " << error.get_msg() << endl;
-		// Revert to a version file
-		setVer = setVersionFile(m_databaseName, version);
+		cerr << "Couldn't set metadata: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (const Xapian::Error &error)
 	{
-		cerr << "Couldn't set database version: " << error.get_type() << ": " << error.get_msg() << endl;
+		cerr << "Couldn't set metadata: " << error.get_type() << ": " << error.get_msg() << endl;
 	}
 	catch (...)
 	{
-		cerr << "Couldn't set database version, unknown exception occured" << endl;
+		cerr << "Couldn't set metadata, unknown exception occured" << endl;
 	}
 	pDatabase->unlock();
+
+	return setMetadata;
 #else
-	setVer = setVersionFile(m_databaseName, version);
+	return false;
 #endif
-
-	// While we are at it, create a CACHEDIR.TAG file
-	// See the spec at http://www.brynosaurus.com/cachedir/
-	string cacheDirFileName(m_databaseName + "/CACHEDIR.TAG");
-	ofstream cacheDirFile;
-	cacheDirFile.open(cacheDirFileName.c_str(), ios::trunc);
-	if (cacheDirFile.good() == true)
-	{
-		cacheDirFile << "Signature: 8a477f597d28d172789f06886806bc55" << endl;
-		cacheDirFile << "# This file is a cache directory tag created by Pinot." << endl;
-		cacheDirFile << "# For information about cache directory tags, see:" << endl;
-		cacheDirFile << "# http://www.brynosaurus.com/cachedir/" << endl;
-	}
-	cacheDirFile.close();
-
-	return setVer;
 }
 
 /// Gets the index location.
@@ -1019,7 +1183,7 @@ bool XapianIndex::setLabels(const set<string> &labels)
 #if ENABLE_XAPIAN_DB_METADATA>0
 	string labelString;
 
-	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName);
+	XapianDatabase *pDatabase = XapianDatabaseFactory::getDatabase(m_databaseName, false);
 	if (pDatabase == NULL)
 	{
 		cerr << "Bad index " << m_databaseName << endl;
@@ -1399,16 +1563,7 @@ bool XapianIndex::setDocumentsLabels(const set<unsigned int> &docIds,
 			}
 
 			// Set new labels
-			for (set<string>::const_iterator labelIter = labels.begin(); labelIter != labels.end();
-				++labelIter)
-			{
-				// Prevent from setting internal labels
-				if ((labelIter->empty() == false) &&
-					(labelIter->substr(0, 2) != "X-"))
-				{
-					doc.add_term(string("XLABEL:") + XapianDatabase::limitTermLength(Url::escapeUrl(*labelIter)));
-				}
-			}
+			addLabelsToDocument(doc, labels, true);
 
 			pIndex->replace_document(docId, doc);
 			updatedLabels = true;
@@ -1662,10 +1817,13 @@ bool XapianIndex::indexDocument(const Document &document, const std::set<std::st
 	unsigned int dataLength = 0;
 	const char *pData = document.getData(dataLength);
 
+	// Don't scan the document if a language is specified
+	m_stemLanguage = Languages::toEnglish(docInfo.getLanguage());
 	if ((pData != NULL) &&
 		(dataLength > 0))
 	{
-		m_stemLanguage = scanDocument(pData, dataLength, docInfo);
+		m_stemLanguage = scanDocument(m_stemLanguage, pData, dataLength);
+		docInfo.setLanguage(Languages::toLocale(m_stemLanguage));
 	}
 
 	try
@@ -1674,26 +1832,24 @@ bool XapianIndex::indexDocument(const Document &document, const std::set<std::st
 		if (pIndex != NULL)
 		{
 			Xapian::Document doc;
+			Xapian::termcount termPos = 0;
 
 			// Populate the Xapian document
-			addCommonTerms(docInfo, doc, *pIndex);
+			addCommonTerms(docInfo, doc, *pIndex, termPos);
 			if ((pData != NULL) &&
 				(dataLength > 0))
 			{
 				Xapian::Utf8Iterator itor(pData, dataLength);
-				addPostingsToDocument(itor, doc, *pIndex, "", false);
-			}
-
-			// Add labels
-			for (set<string>::const_iterator labelIter = labels.begin(); labelIter != labels.end();
-				++labelIter)
-			{
-				doc.add_term(string("XLABEL:") + XapianDatabase::limitTermLength(Url::escapeUrl(*labelIter)));
+				addPostingsToDocument(itor, doc, *pIndex, "",
+					false, m_doSpelling, termPos);
 			}
 #ifdef DEBUG
-			cout << "XapianIndex::indexDocument: " << labels.size() << " labels" << endl;
+			cout << "XapianIndex::indexDocument: " << labels.size() << " labels for URL " << docInfo.getLocation() << endl;
 #endif
 
+			// Add labels
+			addLabelsToDocument(doc, labels, false);
+
 			// Set data
 			setDocumentData(docInfo, doc, m_stemLanguage);
 
@@ -1739,13 +1895,11 @@ bool XapianIndex::updateDocument(unsigned int docId, const Document &document)
 
 	// Don't scan the document if a language is specified
 	m_stemLanguage = Languages::toEnglish(docInfo.getLanguage());
-	if (m_stemLanguage.empty() == true)
+	if ((pData != NULL) &&
+		(dataLength > 0))
 	{
-		if ((pData != NULL) &&
-			(dataLength > 0))
-		{
-			m_stemLanguage = scanDocument(pData, dataLength, docInfo);
-		}
+		m_stemLanguage = scanDocument(m_stemLanguage, pData, dataLength);
+		docInfo.setLanguage(Languages::toLocale(m_stemLanguage));
 	}
 
 	Xapian::WritableDatabase *pIndex = NULL;
@@ -1761,22 +1915,20 @@ bool XapianIndex::updateDocument(unsigned int docId, const Document &document)
 		if (pIndex != NULL)
 		{
 			Xapian::Document doc;
+			Xapian::termcount termPos = 0;
 
 			// Populate the Xapian document
-			addCommonTerms(docInfo, doc, *pIndex);
+			addCommonTerms(docInfo, doc, *pIndex, termPos);
 			if ((pData != NULL) &&
 				(dataLength > 0))
 			{
 				Xapian::Utf8Iterator itor(pData, dataLength);
-				addPostingsToDocument(itor, doc, *pIndex, "", false);
+				addPostingsToDocument(itor, doc, *pIndex, "",
+					false, m_doSpelling, termPos);
 			}
 
 			// Add labels
-			for (set<string>::const_iterator labelIter = labels.begin(); labelIter != labels.end();
-				++labelIter)
-			{
-				doc.add_term(string("XLABEL:") + XapianDatabase::limitTermLength(Url::escapeUrl(*labelIter)));
-			}
+			addLabelsToDocument(doc, labels, false);
 
 			// Set data
 			setDocumentData(docInfo, doc, m_stemLanguage);
@@ -1825,11 +1977,12 @@ bool XapianIndex::updateDocumentInfo(unsigned int docId, const DocumentInfo &doc
 		if (pIndex != NULL)
 		{
 			Xapian::Document doc = pIndex->get_document(docId);
+			Xapian::termcount termPos = 0;
 
 			// Update the document data with the current language
 			m_stemLanguage = Languages::toEnglish(docInfo.getLanguage());
 			removeCommonTerms(doc, *pIndex);
-			addCommonTerms(docInfo, doc, *pIndex);
+			addCommonTerms(docInfo, doc, *pIndex, termPos);
 			setDocumentData(docInfo, doc, m_stemLanguage);
 
 			pIndex->replace_document(docId, doc);
