@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2006 Fabrice Colin
+ *  Copyright 2005-2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,7 +26,10 @@
 #include "SQLiteBase.h"
 
 using std::cout;
+using std::cerr;
 using std::endl;
+using std::string;
+using std::vector;
 
 static int busyHandler(void *pData, int lockNum)
 {
@@ -34,14 +37,10 @@ static int busyHandler(void *pData, int lockNum)
 	return 1;
 }
 
-SQLiteRow::SQLiteRow(const vector<string> &rowColumns, int nColumns) :
-	m_nColumns(nColumns)
+SQLiteRow::SQLiteRow(const vector<string> &rowColumns, unsigned int nColumns) :
+	SQLRow(nColumns)
 {
-	if (rowColumns.empty() == true)
-	{
-		m_nColumns = 0;
-	}
-	else
+	if (rowColumns.empty() == false)
 	{
 		m_columns.reserve(rowColumns.size());
 #if 0
@@ -60,12 +59,7 @@ SQLiteRow::~SQLiteRow()
 {
 }
 
-int SQLiteRow::getColumnsCount(void) const
-{
-	return m_nColumns;
-}
-
-string SQLiteRow::getColumn(int nColumn) const
+string SQLiteRow::getColumn(unsigned int nColumn) const
 {
 	if (nColumn < m_nColumns)
 	{
@@ -74,7 +68,8 @@ string SQLiteRow::getColumn(int nColumn) const
 		{
 			if (i == nColumn)
 			{
-				string column = *colIter;
+				string column(*colIter);
+
 				return column;
 			}
 			++colIter;
@@ -84,17 +79,16 @@ string SQLiteRow::getColumn(int nColumn) const
 	return "";
 }
 
-SQLiteResults::SQLiteResults(char **results, int nRows, int nColumns) :
-	m_results(results),
-	m_nRows(nRows),
-	m_nColumns(nColumns),
-	m_nCurrentRow(0)
+SQLiteResults::SQLiteResults(char **results, unsigned long nRows, unsigned int nColumns) :
+	SQLResults(nRows, nColumns),
+	m_results(results)
 {
 	// Check we actually have results
 	if ((m_results == NULL) ||
 		(m_nRows <= 0))
 	{
-		m_nRows = m_nColumns = m_nCurrentRow = 0;
+		m_nRows = m_nCurrentRow = 0;
+		m_nColumns = 0;
 	}
 }
 
@@ -103,18 +97,7 @@ SQLiteResults::~SQLiteResults()
 	sqlite3_free_table(m_results);
 }
 
-bool SQLiteResults::hasMoreRows(void) const
-{
-	if ((m_nCurrentRow >= 0) &&
-		(m_nCurrentRow < m_nRows))
-	{
-		return true;
-	}
-
-	return false;
-}
-
-string SQLiteResults::getColumnName(int nColumn) const
+string SQLiteResults::getColumnName(unsigned int nColumn) const
 {
 	if (nColumn < m_nColumns)
 	{
@@ -124,7 +107,7 @@ string SQLiteResults::getColumnName(int nColumn) const
 	return "";
 }
 
-SQLiteRow *SQLiteResults::nextRow(void)
+SQLRow *SQLiteResults::nextRow(void)
 {
 	if ((m_nCurrentRow < 0) ||
 		(m_nCurrentRow >= m_nRows))
@@ -133,11 +116,11 @@ SQLiteRow *SQLiteResults::nextRow(void)
 	}
 
 	// The very first row holds the column names
-	unsigned int firstIndex = (m_nCurrentRow  + 1) * m_nColumns;
-	unsigned int lastIndex = firstIndex + m_nColumns - 1;
+	unsigned long firstIndex = (m_nCurrentRow  + 1) * m_nColumns;
+	unsigned long lastIndex = firstIndex + m_nColumns - 1;
 	vector<string> rowColumns;
 
-	for (unsigned int i = firstIndex; i <= lastIndex; ++i)
+	for (unsigned long i = firstIndex; i <= lastIndex; ++i)
 	{
 		if (m_results[i] == NULL)
 		{
@@ -153,21 +136,14 @@ SQLiteRow *SQLiteResults::nextRow(void)
 	return new SQLiteRow(rowColumns, m_nColumns);
 }
 
-bool SQLiteResults::reset(void)
-{
-	m_nCurrentRow = 0;
-
-	return true;
-}
-
-SQLiteBase::SQLiteBase(const string &database, bool onDemand) :
-	m_databaseName(database),
+SQLiteBase::SQLiteBase(const string &databaseName, bool onDemand) :
+	SQLDB(databaseName),
 	m_onDemand(onDemand),
 	m_pDatabase(NULL)
 {
 	if (m_onDemand == false)
 	{
-		open(m_databaseName);
+		open();
 	}
 }
 
@@ -179,38 +155,38 @@ SQLiteBase::~SQLiteBase()
 	}
 }
 
-bool SQLiteBase::check(const string &database)
+bool SQLiteBase::check(const string &databaseName)
 {
 	struct stat dbStat;
 
 	// The specified path must be a file
-	if ((stat(database.c_str(), &dbStat) != -1) &&
+	if ((stat(databaseName.c_str(), &dbStat) != -1) &&
 		(!S_ISREG(dbStat.st_mode)))
 	{
 		// It exists, but it's not a file as expected
-		cerr << "SQLiteBase::check: " << database << " is not a file" << endl;
+		cerr << databaseName << " is not a file" << endl;
 		return false;
 	}
 
 	return true;
 }
 
-void SQLiteBase::open(const string &database)
+void SQLiteBase::open(void)
 {
 	// Open the new database
-	if (sqlite3_open(database.c_str(), &m_pDatabase) != SQLITE_OK)
+	if (sqlite3_open(m_databaseName.c_str(), &m_pDatabase) != SQLITE_OK)
 	{
 		// An handle is returned even when an error occurs !
 		if (m_pDatabase != NULL)
 		{
-			cerr << "SQLiteBase::open: " << sqlite3_errmsg(m_pDatabase) << endl;
+			cerr << sqlite3_errmsg(m_pDatabase) << endl;
 			close();
 			m_pDatabase = NULL;
 		}
 	}
 	else if (m_pDatabase == NULL)
 	{
-		cerr << "SQLiteBase::open: couldn't open " << database << endl;
+		cerr << "Couldn't open " << m_databaseName << endl;
 	}
 	else
 	{
@@ -227,6 +203,16 @@ void SQLiteBase::close(void)
 	}
 }
 
+bool SQLiteBase::isOpen(void) const
+{
+	if (m_pDatabase == NULL)
+	{
+		return false;
+	}
+
+	return true;
+}
+
 bool SQLiteBase::executeSimpleStatement(const string &sql)
 {
 	char *errMsg = NULL;
@@ -239,7 +225,7 @@ bool SQLiteBase::executeSimpleStatement(const string &sql)
 
 	if (m_onDemand == true)
 	{
-		open(m_databaseName);
+		open();
 	}
 	if (m_pDatabase == NULL)
 	{
@@ -253,7 +239,7 @@ bool SQLiteBase::executeSimpleStatement(const string &sql)
 	{
 		if (errMsg != NULL)
 		{
-			cerr << "SQLiteBase::executeSimpleStatement: statement <" << sql << "> failed: " << errMsg << endl;
+			cerr << "Statement <" << sql << "> failed: " << errMsg << endl;
 
 			sqlite3_free(errMsg);
 		}
@@ -268,7 +254,7 @@ bool SQLiteBase::executeSimpleStatement(const string &sql)
 	return success;
 }
 
-SQLiteResults *SQLiteBase::executeStatement(const char *sqlFormat, ...)
+SQLResults *SQLiteBase::executeStatement(const char *sqlFormat, ...)
 {
 	SQLiteResults *pResults = NULL;
 #ifdef _USE_VSNPRINTF
@@ -283,7 +269,7 @@ SQLiteResults *SQLiteBase::executeStatement(const char *sqlFormat, ...)
 
 	if (m_onDemand == true)
 	{
-		open(m_databaseName);
+		open();
 	}
 	if (m_pDatabase == NULL)
 	{
@@ -344,14 +330,14 @@ SQLiteResults *SQLiteBase::executeStatement(const char *sqlFormat, ...)
 	{
 		if (errMsg != NULL)
 		{
-			cerr << "SQLiteBase::executeStatement: statement <" << stringBuff << "> failed: " << errMsg << endl;
+			cerr << "Statement <" << stringBuff << "> failed: " << errMsg << endl;
 
 			sqlite3_free(errMsg);
 		}
 	}
 	else
 	{
-		pResults = new SQLiteResults(results, nRows, nColumns);
+		pResults = new SQLiteResults(results, (unsigned long)nRows, (unsigned int)nColumns);
 	}
 	va_end(ap);
 #ifndef _USE_VSNPRINTF
