@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2006 Fabrice Colin
+ *  Copyright 2005-2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,45 +24,35 @@
 
 #include <sqlite3.h>
 
-using namespace std;
+#include "SQLDB.h"
 
 /// A row in a SQLite table.
-class SQLiteRow
+class SQLiteRow : public SQLRow
 {
 	public:
-		SQLiteRow(const vector<string> &rowColumns, int nColumns);
+		SQLiteRow(const std::vector<std::string> &rowColumns, unsigned int nColumns);
 		virtual ~SQLiteRow();
 
-		int getColumnsCount(void) const;
-
-		string getColumn(int nColumn) const;
+		virtual std::string getColumn(unsigned int nColumn) const;
 
 	protected:
-		vector<string> m_columns;
-		int m_nColumns;
+		std::vector<std::string> m_columns;
 
 };
 
-/// Results extracted from a SQLite table.
-class SQLiteResults
+/// Results extracted from a SQLite database.
+class SQLiteResults : public SQLResults
 {
 	public:
-		SQLiteResults(char **results, int nRows, int nColumns);
+		SQLiteResults(char **results, unsigned long nRows, unsigned int nColumns);
 		virtual ~SQLiteResults();
 
-		bool hasMoreRows(void) const;
+		virtual std::string getColumnName(unsigned int nColumn) const;
 
-		string getColumnName(int nColumn) const;
-
-		SQLiteRow *nextRow(void);
-
-		bool reset(void);
+		virtual SQLRow *nextRow(void);
 
 	protected:
 		char **m_results;
-		int m_nRows;
-		int m_nColumns;
-		int m_nCurrentRow;
 
 	private:
 		SQLiteResults(const SQLiteResults &other);
@@ -71,23 +61,25 @@ class SQLiteResults
 };
 
 /// Simple C++ wrapper around the SQLite API.
-class SQLiteBase
+class SQLiteBase : public SQLDB
 {
 	public:
-		SQLiteBase(const string &database, bool onDemand = true);
+		SQLiteBase(const std::string &databaseName, bool onDemand = true);
 		virtual ~SQLiteBase();
 
-		static bool check(const string &database);
+		static bool check(const std::string &databaseName);
 
-		bool executeSimpleStatement(const string &sql);
-		SQLiteResults *executeStatement(const char *sqlFormat, ...);
+		virtual bool isOpen(void) const;
+
+		virtual bool executeSimpleStatement(const std::string &sql);
+
+		virtual SQLResults *executeStatement(const char *sqlFormat, ...);
 
 	protected:
-		string m_databaseName;
 		bool m_onDemand;
 		sqlite3 *m_pDatabase;
 
-		void open(const string &database);
+		void open(void);
 		void close(void);
 
 	private:
