@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2006 Fabrice Colin
+ *  Copyright 2005-2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,10 +20,15 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <iostream>
+#include <cstdlib>
 
 #include "Url.h"
 #include "ViewHistory.h"
 
+using std::cout;
+using std::endl;
+using std::string;
+
 ViewHistory::ViewHistory(const string &database) :
 	SQLiteBase(database)
 {
@@ -88,7 +93,7 @@ bool ViewHistory::insertItem(const string &url)
 {
 	bool success = false;
 
-	SQLiteResults *results = executeStatement("INSERT INTO ViewHistory \
+	SQLResults *results = executeStatement("INSERT INTO ViewHistory \
 		VALUES('%q', '1', '%d');", Url::escapeUrl(url).c_str(), time(NULL));
 	if (results != NULL)
 	{
@@ -104,11 +109,11 @@ bool ViewHistory::hasItem(const string &url)
 {
 	bool success = false;
 
-	SQLiteResults *results = executeStatement("SELECT Url FROM ViewHistory \
+	SQLResults *results = executeStatement("SELECT Url FROM ViewHistory \
 		WHERE Url='%q';", Url::escapeUrl(url).c_str());
 	if (results != NULL)
 	{
-		SQLiteRow *row = results->nextRow();
+		SQLRow *row = results->nextRow();
 		if (row != NULL)
 		{
 			// If this returns anything, it's the URL we are looking for
@@ -131,10 +136,10 @@ unsigned int ViewHistory::getItemsCount(void)
 {
 	unsigned int count = 0;
 
-	SQLiteResults *results = executeStatement("SELECT COUNT(*) FROM ViewHistory;");
+	SQLResults *results = executeStatement("SELECT COUNT(*) FROM ViewHistory;");
 	if (results != NULL)
 	{
-		SQLiteRow *row = results->nextRow();
+		SQLRow *row = results->nextRow();
 		if (row != NULL)
 		{
 			count = atoi(row->getColumn(0).c_str());
@@ -153,7 +158,7 @@ bool ViewHistory::deleteItem(const string &url)
 {
 	bool success = false;
 
-	SQLiteResults *results = executeStatement("DELETE FROM ViewHistory \
+	SQLResults *results = executeStatement("DELETE FROM ViewHistory \
 		WHERE Url='%q';", Url::escapeUrl(url).c_str());
 	if (results != NULL)
 	{
@@ -169,7 +174,7 @@ bool ViewHistory::expireItems(time_t expiryDate)
 {
 	bool success = false;
 
-	SQLiteResults *results = executeStatement("DELETE FROM ViewHistory \
+	SQLResults *results = executeStatement("DELETE FROM ViewHistory \
 		WHERE Date<'%d';", expiryDate);
 	if (results != NULL)
 	{
