@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007 Fabrice Colin
+ *  Copyright 2007,2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include "StringManip.h"
 #include "TimeConverter.h"
 #include "Url.h"
+#include "TextConverter.h"
 #include "filters/FilterFactory.h"
 #include "FilterUtils.h"
 
@@ -163,9 +164,6 @@ bool FilterUtils::feedFilter(const Document &doc, Dijon::Filter *pFilter)
 	if (((dataLength > 0) && (pData != NULL)) &&
 		(pFilter->is_data_input_ok(Dijon::Filter::DOCUMENT_DATA) == true))
 	{
-#ifdef DEBUG
-		cout << "FilterUtils::feedFilter: feeding " << dataLength << " bytes of data from " << location << endl;
-#endif
 		fedInput = pFilter->set_document_data(pData, dataLength);
 	}
 	// ... to feeding the file
@@ -249,24 +247,21 @@ bool FilterUtils::feedFilter(const Document &doc, Dijon::Filter *pFilter)
 
 bool FilterUtils::populateDocument(Document &doc, Dijon::Filter *pFilter)
 {
-	string uri, ipath;
+	string charset, uri, ipath;
 
 	if (pFilter == NULL)
 	{
 		return false;
 	}
 
+	// Go through the whole thing
 	const map<string, string> &metaData = pFilter->get_meta_data();
 	for (map<string, string>::const_iterator metaIter = metaData.begin();
 		metaIter != metaData.end(); ++metaIter)
 	{
-		if (metaIter->first == "content")
+		if (metaIter->first == "charset")
 		{
-			doc.setData(metaIter->second.c_str(), metaIter->second.length());
-#ifdef DEBUG
-			cout << "FilterUtils::populateDocument: set " << metaIter->second.length()
-				<< " characters of data" << endl;
-#endif
+			charset = metaIter->second;
 		}
 		else if (metaIter->first == "date")
 		{
@@ -296,10 +291,6 @@ bool FilterUtils::populateDocument(Document &doc, Dijon::Filter *pFilter)
 			else cout << "FilterUtils::populateDocument: ignoring size zero" << endl;
 #endif
 		}
-		else if (metaIter->first == "title")
-		{
-			doc.setTitle(metaIter->second);
-		}
 		else if (metaIter->first == "uri")
 		{
 			uri = metaIter->second;
@@ -319,7 +310,35 @@ bool FilterUtils::populateDocument(Document &doc, Dijon::Filter *pFilter)
 	}
 	if (ipath.empty() == false)
 	{
-		doc.setLocation(doc.getLocation() + "?" + ipath);
+		string location(doc.getLocation());
+
+		location += "?";
+		location += ipath;
+		doc.setLocation(location);
+	}
+
+	TextConverter converter(20);
+
+	// Content and title may have to be converted
+	map<string, string>::const_iterator contentIter = metaData.find("content");
+	if (contentIter != metaData.end())
+	{
+		string utf8Data(converter.toUTF8(contentIter->second, charset));
+
+		if (converter.getErrorsCount() > 0)
+		{
+			cerr << doc.getLocation() << " may not have been fully converted to UTF-8" << endl;
+		}
+		doc.setData(utf8Data.c_str(), utf8Data.length());
+#ifdef DEBUG
+		cout << "FilterUtils::populateDocument: set " << utf8Data.length() << "/" << contentIter->second.length()
+			<< " bytes, converted from charset " << charset << endl;
+#endif
+	}
+	contentIter = metaData.find("title");
+	if (contentIter != metaData.end())
+	{
+		doc.setTitle(converter.toUTF8(contentIter->second, charset));
 	}
 
 	return true;
