@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007 Fabrice Colin
+ *  Copyright 2007,2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -36,16 +36,19 @@
 #endif
 #endif
 #include <unistd.h>
+#include <sstream>
 #include <iostream>
+#include <cstring>
 #include <libxml/xmlreader.h>
 
 #include "ExternalFilter.h"
 
-using std::string;
-using std::set;
-using std::map;
 using std::cout;
 using std::endl;
+using std::string;
+using std::stringstream;
+using std::set;
+using std::map;
 
 using namespace Dijon;
 
@@ -113,6 +116,7 @@ static string shell_protect(const string &file_name)
 
 map<string, string> ExternalFilter::m_commandsByType;
 map<string, string> ExternalFilter::m_outputsByType;
+map<string, string> ExternalFilter::m_charsetsByType;
 
 ExternalFilter::ExternalFilter(const string &mime_type) :
 	Filter(mime_type),
@@ -198,6 +202,12 @@ bool ExternalFilter::next_document(void)
 			{
 				m_metaData["mimetype"] = outputIter->second;
 			}
+			// Is it in a known charset ?
+			map<string, string>::const_iterator charsetIter = m_charsetsByType.find(m_mimeType);
+			if (charsetIter != m_charsetsByType.end())
+			{
+				m_metaData["charset"] = charsetIter->second;
+			}
 
 			return true;
 		}
@@ -260,7 +270,7 @@ void ExternalFilter::initialize(const std::string &config_file, set<std::string>
 		// Get all filter elements
 		if (xmlStrncmp(pCurrentNode->name, BAD_CAST"filter", 6) == 0)
 		{
-			string mimeType, command, arguments, output;
+			string mimeType, charset, command, arguments, output;
 
 			for (xmlNode *pCurrentCodecNode = pCurrentNode->children;
 				pCurrentCodecNode != NULL; pCurrentCodecNode = pCurrentCodecNode->next)
@@ -281,6 +291,10 @@ void ExternalFilter::initialize(const std::string &config_file, set<std::string>
 				{
 					mimeType = pChildContent;
 				}
+				else if (xmlStrncmp(pCurrentCodecNode->name, BAD_CAST"charset", 7) == 0)
+				{
+					charset = pChildContent;
+				}
 				else if (xmlStrncmp(pCurrentCodecNode->name, BAD_CAST"command", 7) == 0)
 				{
 					command = pChildContent;
@@ -314,6 +328,11 @@ void ExternalFilter::initialize(const std::string &config_file, set<std::string>
 				{
 					m_outputsByType[mimeType] = output;
 				}
+				// Charset
+				if (charset.empty() == false)
+				{
+					m_charsetsByType[mimeType] = charset;
+				}
 
 				types.insert(mimeType);
 			}
@@ -395,13 +414,12 @@ bool ExternalFilter::run_command(const string &command)
 				ssize_t bytesRead = read(outFd, (void*)fileBuffer, outStats.st_size);
 				if (bytesRead > 0)
 				{
-					char numStr[64];
-
 					fileBuffer[bytesRead] = '\0';
 
 					m_metaData["content"] = string(fileBuffer, (unsigned int)bytesRead);
-					snprintf(numStr, 64, "%d", outStats.st_size);
-					m_metaData["size"] = numStr;
+					stringstream numStream;
+					numStream << outStats.st_size;
+					m_metaData["size"] = numStream.str();
 					gotOutput = true;
 				}
 #ifdef DEBUG
@@ -532,9 +550,9 @@ bool ExternalFilter::run_command(const string &command)
 #endif
 
 	m_metaData["content"] = fileBuffer;
-	char numStr[64];
-	snprintf(numStr, 64, "%d", totalSize);
-	m_metaData["size"] = numStr;
+	stringstream numStream;
+	numStream << totalSize;
+	m_metaData["size"] = numStream.str();
 #endif
 
 	return true;
