@@ -34,6 +34,7 @@ using namespace Dijon;
 
 Filter::Filter(const string &mime_type) :
 	m_mimeType(mime_type),
+	m_convertToUTF8Func(NULL),
 	m_deleteInputFile(false)
 {
 }
@@ -62,6 +63,11 @@ string Filter::get_mime_type(void) const
 	return m_mimeType;
 }
 
+bool Filter::set_utf8_converter(convert_to_utf8_func *func)
+{
+	m_convertToUTF8Func = func;
+}
+
 const map<string, string> &Filter::get_meta_data(void) const
 {
 	return m_metaData;
