@@ -139,7 +139,7 @@ namespace Dijon
 			bool m_isValid;
 			bool m_findAbstract;
 			unsigned int m_textPos;
-			std::string m_lastHash;
+			std::string m_lastText;
 			bool m_inHead;
 			bool m_foundHead;
 			bool m_appendToTitle;
@@ -164,7 +164,7 @@ namespace Dijon
 
 	virtual void rewind(void);
 
-	bool parse_html(const std::string &html_doc);
+	bool parse_html(const char *pData, unsigned int dataLen);
 
     private:
 	/// HtmlFilter objects cannot be copied.
