@@ -26,6 +26,7 @@
 #include <fnmatch.h>
 #include <algorithm>
 #include <iostream>
+#include <cstring>
 
 #include <glibmm/convert.h>
 #include <glibmm/date.h>
@@ -119,6 +120,7 @@ PinotSettings::PinotSettings() :
 		// No, create it then
 		if (mkdir(directoryName.c_str(), (mode_t)S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IWGRP|S_IXGRP|S_IROTH|S_IXOTH) == 0)
 		{
+			cerr << "Created directory " << directoryName << endl;
 			m_firstRun = true;
 		}
 		else
@@ -250,9 +252,6 @@ string PinotSettings::getHistoryDatabaseName(bool needToQueryDaemonHistory)
 	{
 		historyDatabase += "/history";
 	}
-#ifdef DEBUG
-	cout << "PinotSettings::getHistoryDatabaseName: " << historyDatabase << endl;
-#endif
 
 	return historyDatabase;
 }
@@ -286,7 +285,7 @@ void PinotSettings::clear(void)
 	m_proxyEnabled = false;
 	m_indexableLocations.clear();
 	m_filePatternsList.clear();
-	m_isBlackList = false;
+	m_isBlackList = true;
 	m_cacheProviders.clear();
 	m_cacheProtocols.clear();
 
@@ -330,32 +329,7 @@ bool PinotSettings::load(void)
 		m_labels.insert(_("Important"));
 		m_labels.insert(_("New"));
 		m_labels.insert(_("Personal"));
-		// Skip common image, video and archive types
-		m_filePatternsList.insert("*~");
-		m_filePatternsList.insert("*.Z");
-		m_filePatternsList.insert("*.a");
-		m_filePatternsList.insert("*.avi");
-		m_filePatternsList.insert("*.asf");
-		m_filePatternsList.insert("*.cap");
-		m_filePatternsList.insert("*.gif");
-		m_filePatternsList.insert("*.iso");
-		m_filePatternsList.insert("*.jpeg");
-		m_filePatternsList.insert("*.jpg");
-		m_filePatternsList.insert("*.la");
-		m_filePatternsList.insert("*.lha");
-		m_filePatternsList.insert("*.mov");
-		m_filePatternsList.insert("*.msf");
-		m_filePatternsList.insert("*.mpeg");
-		m_filePatternsList.insert("*.mpg");
-		m_filePatternsList.insert("*.o");
-		m_filePatternsList.insert("*.png");
-		m_filePatternsList.insert("*.rar");
-		m_filePatternsList.insert("*.sh");
-		m_filePatternsList.insert("*.so");
-		m_filePatternsList.insert("*.tiff");
-		m_filePatternsList.insert("*.wmv");
-		m_filePatternsList.insert("*.xbm");
-		m_filePatternsList.insert("*.xpm");
+		m_isBlackList = getDefaultPatterns(m_filePatternsList);
 		// Create default queries
 		struct passwd *pPasswd = getpwuid(geteuid());
 		if (pPasswd != NULL)
@@ -413,7 +387,10 @@ bool PinotSettings::loadConfiguration(const std::string &fileName, bool isGlobal
 		(!S_ISREG(fileStat.st_mode)))
 	{
 		cerr << "Couldn't open configuration file " << fileName << endl;
-		m_firstRun = true;
+		if (isGlobal == false)
+		{
+			m_firstRun = true;
+		}
 		return false;
 	}
 
@@ -883,13 +860,17 @@ bool PinotSettings::loadQueries(const Element *pElem)
 		}
 		else if (nodeName == "index")
 		{
-			if (nodeContent == "ALL")
+			if (nodeContent == "NEW")
 			{
-				queryProps.setIndexResults(true);
+				queryProps.setIndexResults(QueryProperties::NEW_RESULTS);
+			}
+			else if (nodeContent == "ALL")
+			{
+				queryProps.setIndexResults(QueryProperties::ALL_RESULTS);
 			}
 			else
 			{
-				queryProps.setIndexResults(false);
+				queryProps.setIndexResults(QueryProperties::NOTHING);
 			}
 		}
 		else if (nodeName == "label")
@@ -1329,15 +1310,15 @@ bool PinotSettings::save(void)
 		{
 			return false;
 		}
-		sprintf(numStr, "%d", m_xPos);
+		snprintf(numStr, 64, "%d", m_xPos);
 		addChildElement(pElem, "xpos", numStr);
-		sprintf(numStr, "%d", m_yPos);
+		snprintf(numStr, 64, "%d", m_yPos);
 		addChildElement(pElem, "ypos", numStr);
-		sprintf(numStr, "%d", m_width);
+		snprintf(numStr, 64, "%d", m_width);
 		addChildElement(pElem, "width", numStr);
-		sprintf(numStr, "%d", m_height);
+		snprintf(numStr, 64, "%d", m_height);
 		addChildElement(pElem, "height", numStr);
-		sprintf(numStr, "%d", m_panePos);
+		snprintf(numStr, 64, "%d", m_panePos);
 		addChildElement(pElem, "panepos", numStr);
 		addChildElement(pElem, "expandqueries", (m_expandQueries ? "YES" : "NO"));
 		addChildElement(pElem, "showengines", (m_showEngines ? "YES" : "NO"));
@@ -1388,9 +1369,21 @@ bool PinotSettings::save(void)
 			addChildElement(pElem, "sortorder", (queryIter->second.getSortOrder() == QueryProperties::DATE ? "DATE" : "RELEVANCE"));
 			addChildElement(pElem, "text", queryIter->second.getFreeQuery());
 			addChildElement(pElem, "stemlanguage", Languages::toEnglish(queryIter->second.getStemmingLanguage()));
-			sprintf(numStr, "%u", queryIter->second.getMaximumResultsCount());
+			snprintf(numStr, 64, "%u", queryIter->second.getMaximumResultsCount());
 			addChildElement(pElem, "maxresults", numStr);
-			addChildElement(pElem, "index", (queryIter->second.getIndexResults() ? "ALL" : "NONE"));
+			QueryProperties::IndexWhat indexResults = queryIter->second.getIndexResults();
+			if (indexResults == QueryProperties::NEW_RESULTS)
+			{
+				addChildElement(pElem, "index", "NEW");
+			}
+			else if (indexResults == QueryProperties::ALL_RESULTS)
+			{
+				addChildElement(pElem, "index", "ALL");
+			}
+			else
+			{
+				addChildElement(pElem, "index", "NONE");
+			}
 			addChildElement(pElem, "label", queryIter->second.getLabelName());
 			addChildElement(pElem, "modified", (queryIter->second.getModified() ? "YES" : "NO"));
 		}
@@ -1414,11 +1407,11 @@ bool PinotSettings::save(void)
 		{
 			return false;
 		}
-		sprintf(numStr, "%u", m_newResultsColourRed);
+		snprintf(numStr, 64, "%u", m_newResultsColourRed);
 		addChildElement(pElem, "red", numStr);
-		sprintf(numStr, "%u", m_newResultsColourGreen);
+		snprintf(numStr, 64, "%u", m_newResultsColourGreen);
 		addChildElement(pElem, "green", numStr);
-		sprintf(numStr, "%u", m_newResultsColourBlue);
+		snprintf(numStr, 64, "%u", m_newResultsColourBlue);
 		addChildElement(pElem, "blue", numStr);
 		// Proxy
 		pElem = pRootElem->add_child("proxy");
@@ -1427,7 +1420,7 @@ bool PinotSettings::save(void)
 			return false;
 		}
 		addChildElement(pElem, "address", m_proxyAddress);
-		sprintf(numStr, "%u", m_proxyPort);
+		snprintf(numStr, 64, "%u", m_proxyPort);
 		addChildElement(pElem, "port", numStr);
 		addChildElement(pElem, "type", m_proxyType);
 		addChildElement(pElem, "enable", (m_proxyEnabled ? "YES" : "NO"));
@@ -1739,6 +1732,42 @@ void PinotSettings::clearQueries(void)
 	m_queries.clear();
 }
 
+/// Gets default patterns.
+bool PinotSettings::getDefaultPatterns(set<ustring> &defaultPatterns)
+{
+	defaultPatterns.clear();
+
+	// Skip common image, video and archive types
+	defaultPatterns.insert("*~");
+	defaultPatterns.insert("*.Z");
+	defaultPatterns.insert("*.a");
+	defaultPatterns.insert("*.asf");
+	defaultPatterns.insert("*.avi");
+	defaultPatterns.insert("*.cap");
+	defaultPatterns.insert("*.divx");
+	defaultPatterns.insert("*.flv");
+	defaultPatterns.insert("*.gif");
+	defaultPatterns.insert("*.iso");
+	defaultPatterns.insert("*.la");
+	defaultPatterns.insert("*.lha");
+	defaultPatterns.insert("*.mov");
+	defaultPatterns.insert("*.msf");
+	defaultPatterns.insert("*.mpeg");
+	defaultPatterns.insert("*.mpg");
+	defaultPatterns.insert("*.mo");
+	defaultPatterns.insert("*.o");
+	defaultPatterns.insert("*.png");
+	defaultPatterns.insert("*.rar");
+	defaultPatterns.insert("*.sh");
+	defaultPatterns.insert("*.so");
+	defaultPatterns.insert("*.tiff");
+	defaultPatterns.insert("*.wmv");
+	defaultPatterns.insert("*.xbm");
+	defaultPatterns.insert("*.xpm");
+
+	return true;
+}
+
 /// Determines if a file matches the blacklist.
 bool PinotSettings::isBlackListed(const string &fileName)
 {
