@@ -20,9 +20,7 @@
 #define _PINOTUTILS_HH
 
 #include <string>
-#if GTKMM_MAJOR_VERSION==2 && GTKMM_MINOR_VERSION>2
-#include <sigc++/compatibility.h>
-#endif
+#include <gtkmmconfig.h>
 #include <glibmm/ustring.h>
 #include <gtkmm/window.h>
 #include <gtkmm/filechooserdialog.h>
