@@ -372,9 +372,9 @@ bool ResultsTree::onSelectionSelect(const RefPtr<TreeModel>& model,
 	const TreeModel::iterator iter = model->get_iter(node_path);
 	const TreeModel::Row row = *iter;
 
-	m_indexNames.clear();
-
-	if (path_currently_selected == false)
+	// In flat mode, don't bother about the extract
+	if ((path_currently_selected == false) &&
+		(m_groupMode != FLAT))
 	{
 		// Clear the extract
 		m_extractTreeView->get_selection()->unselect_all();
@@ -486,9 +486,9 @@ bool ResultsTree::addResults(const string &engineName, const vector<DocumentInfo
 		{
 			// Yes, it is
 			indexId = m_settings.getIndexIdByName(engineName);
-			engineId = m_settings.getEngineId("Xapian");
+			engineId = m_settings.getEngineId(m_settings.m_defaultBackend);
 #ifdef DEBUG
-			cout << "ResultsTree::addResults: engine is index " << engineName << " " << indexId << endl;
+			cout << "ResultsTree::addResults: engine is index " << engineName << " " << indexId << " " << engineId << endl;
 #endif
 		}
 #ifdef DEBUG
@@ -496,7 +496,7 @@ bool ResultsTree::addResults(const string &engineName, const vector<DocumentInfo
 #endif
 	}
 #ifdef DEBUG
-	cout << "ResultsTree::addResults: ID for engine " << engineName << " is " << engineId <<  endl;
+	else cout << "ResultsTree::addResults: ID for engine " << engineName << " is " << engineId <<  endl;
 #endif
 
 	QueryHistory queryHistory(m_settings.getHistoryDatabaseName());
@@ -578,7 +578,9 @@ bool ResultsTree::addResults(const string &engineName, const vector<DocumentInfo
 
 		++count;
 
-		unsigned int docId = resultIter->getIsIndexed(indexId);
+		// We already got indexId from PinotSettings::getIndexIdByName()
+		unsigned int docIndexId = 0;
+		unsigned int docId = resultIter->getIsIndexed(docIndexId);
 		bool isIndexed = false;
 
 		if (docId > 0)
@@ -597,8 +599,8 @@ bool ResultsTree::addResults(const string &engineName, const vector<DocumentInfo
 		{
 #ifdef DEBUG
 			cout << "ResultsTree::addResults: added row for result " << count
-				<< ", " << currentScore << ", " << isIndexed << " " << indexId
-				<< " " << resultIter->getTimestamp() << endl;
+				<< ", " << currentScore << ", " << isIndexed << " " << docId
+				<< " " << indexId << " " << resultIter->getTimestamp() << endl;
 #endif
 
 			if (groupIter)
@@ -762,12 +764,12 @@ void ResultsTree::setGroupMode(GroupByMode groupMode)
 					cout << "ResultsTree::setGroupMode: row is for " << engineNames.size() << " engine(s)" << endl;
 #endif
 					// Are there indexes in the list ?
-					set<string>::iterator xapianIter = engineNames.find("Xapian");
-					if ((xapianIter != engineNames.end()) &&
+					set<string>::iterator backendIter = engineNames.find(m_settings.m_defaultBackend);
+					if ((backendIter != engineNames.end()) &&
 						(indexIds > 0))
 					{
 						// Erase this
-						engineNames.erase(xapianIter);
+						engineNames.erase(backendIter);
 #ifdef DEBUG
 						cout << "ResultsTree::setGroupMode: row is for index(es)" << indexIds << endl;
 #endif
@@ -799,7 +801,7 @@ void ResultsTree::setGroupMode(GroupByMode groupMode)
 							indexId = m_settings.getIndexIdByName(engineName);
 							if (indexId > 0)
 							{
-								engineId = m_settings.getEngineId("Xapian");
+								engineId = m_settings.getEngineId(m_settings.m_defaultBackend);
 							}
 #ifdef DEBUG
 							cout << "ResultsTree::setGroupMode: index " << indexId << endl;
@@ -1091,7 +1093,7 @@ bool ResultsTree::deleteResults(const string &engineName)
 		{
 			// Yes, it is
 			indexId = m_settings.getIndexIdByName(engineName);
-			engineId = m_settings.getEngineId("Xapian");
+			engineId = m_settings.getEngineId(m_settings.m_defaultBackend);
 		}
 	}
 
@@ -1295,7 +1297,7 @@ void ResultsTree::exportResults(const string &fileName, bool csvFormat)
 			{
 				// Get the first engine this result was obtained from
 				engineName = *engineNames.begin();
-				if (engineName == "Xapian")
+				if (engineName == m_settings.m_defaultBackend)
 				{
 					m_settings.getIndexNames(indexIds, indexNames);
 					if (indexNames.empty() == false)
@@ -1531,37 +1533,42 @@ ustring ResultsTree::findResultsExtract(const Gtk::TreeModel::Row &row)
 	unsigned int indexIds = row[m_resultsColumns.m_indexes];
 
 #ifdef DEBUG
-	cout << "ResultsTree::findResultsExtract: engines " << engineIds << ", indexes " << indexIds << endl;
+	cout << "ResultsTree::findResultsExtract: " << url << " has engines " << engineIds << ", indexes " << indexIds << endl;
 #endif
 	m_settings.getEngineNames(engineIds, engineNames);
-	if (engineNames.empty() == false)
+	for (set<string>::const_iterator engineIter = engineNames.begin();
+		engineIter != engineNames.end(); ++engineIter)
 	{
-		// Get the first engine this result was obtained from
-		string engineName(*engineNames.begin());
-		if (engineName == "Xapian")
+		string engineName(*engineIter);
+
+		indexNames.clear();
+		if (engineName == m_settings.m_defaultBackend)
 		{
 			m_settings.getIndexNames(indexIds, indexNames);
-			if (indexNames.empty() == false)
-			{
-				// Use the name of the first index as engine name
-				engineName = (*indexNames.begin());
-
-				// Any internal index in there ?
-				for (set<string>::iterator indexIter = indexNames.begin(); indexIter != indexNames.end(); ++indexIter)
-				{
-					if  (m_settings.isInternalIndex(*indexIter) == true)
-					{
-						m_indexNames.insert(*indexIter);
-					}
-				}
-			}
+		}
+		else
+		{
+			// That's not an index but pretend it is
+			indexNames.insert(engineName);
 		}
 
+		for (set<string>::const_iterator indexIter = indexNames.begin();
+			indexIter != indexNames.end(); ++indexIter)
+		{
+			// Use the name of this index as engine name
+			engineName = (*indexNames.begin());
+
 #ifdef DEBUG
-		cout << "ResultsTree::findResultsExtract: first engine for " << url << " was " << engineName << endl;
+			cout << "ResultsTree::findResultsExtract: engine or index " << engineName << endl;
 #endif
-		extract = queryHistory.getItemExtract(from_utf8(m_treeName), engineName, url, charset);
+			extract = queryHistory.getItemExtract(from_utf8(m_treeName), engineName, url, charset);
+			if (extract.empty() == false)
+			{
+				// Stop here
+				return to_utf8(extract, charset);
+			}
+		}
 	}
 
-	return to_utf8(extract, charset);
+	return "";
 }
