@@ -24,6 +24,7 @@
 #include <fcntl.h>
 #include <string.h>
 #include <signal.h>
+#include <time.h>
 #include <errno.h>
 #include <exception>
 #include <iostream>
@@ -58,11 +59,11 @@ using namespace std;
 struct StopThreadFunc
 {
 public:
-	void operator()(map<WorkerThread *, Thread *>::value_type &p)
+	void operator()(map<unsigned int, WorkerThread *>::value_type &p)
 	{
-		p.first->stop();
+		p.second->stop();
 #ifdef DEBUG
-		cout << "StopThreadFunc: stopped thread " << p.first->getId() << endl;
+		cout << "StopThreadFunc: stopped thread " << p.second->getId() << endl;
 #endif
 		Thread::yield();
 	}
@@ -187,11 +188,9 @@ Glib::Thread *WorkerThread::start(void)
 	return Thread::create(sigc::mem_fun(*this, &WorkerThread::threadHandler), false);
 }
 
-bool WorkerThread::stop(void)
+void WorkerThread::stop(void)
 {
 	m_stopped = m_done = true;
-
-	return true;
 }
 
 bool WorkerThread::isStopped(void) const
@@ -347,14 +346,14 @@ WorkerThread *ThreadsManager::get_thread(void)
 	WorkerThread *pWorkerThread = NULL;
 
 	// Get the first thread that's finished
-	if (read_lock_threads() == true)
+	if (write_lock_threads() == true)
 	{
-		for (map<WorkerThread *, Thread *>::iterator threadIter = m_threads.begin();
+		for (map<unsigned int, WorkerThread *>::iterator threadIter = m_threads.begin();
 			threadIter != m_threads.end(); ++threadIter)
 		{
-			unsigned int threadId = threadIter->first->getId();
+			unsigned int threadId = threadIter->first;
 
-			if (threadIter->first->isDone() == false)
+			if (threadIter->second->isDone() == false)
 			{
 #ifdef DEBUG
 				cout << "ThreadsManager::get_thread: thread "
@@ -362,11 +361,11 @@ WorkerThread *ThreadsManager::get_thread(void)
 #endif
 
 				// Foreground threads ought not to run very long
-				if ((threadIter->first->isBackground() == false) &&
-					(threadIter->first->getStartTime() + 300 < timeNow))
+				if ((threadIter->second->isBackground() == false) &&
+					(threadIter->second->getStartTime() + 300 < timeNow))
 				{
 					// This thread has been running for more than 5 minutes already !
-					threadIter->first->stop();
+					threadIter->second->stop();
 
 					cerr << "Stopped long-running thread " << threadId << endl;
 				}
@@ -374,7 +373,7 @@ WorkerThread *ThreadsManager::get_thread(void)
 			else
 			{
 				// This one will do...
-				pWorkerThread = threadIter->first;
+				pWorkerThread = threadIter->second;
 				// Remove it
 				m_threads.erase(threadIter);
 #ifdef DEBUG
@@ -485,6 +484,8 @@ void ThreadsManager::clear_queues(void)
 
 bool ThreadsManager::start_thread(WorkerThread *pWorkerThread, bool inBackground)
 {
+	bool createdThread = false;
+
 	if (pWorkerThread == NULL)
 	{
 		return false;
@@ -505,25 +506,44 @@ bool ThreadsManager::start_thread(WorkerThread *pWorkerThread, bool inBackground
 			<< " will run in the foreground" << endl;
 #endif
 
-	// Start the thread
-	Thread *pThread = pWorkerThread->start();
-	if (pThread == NULL)
-	{
-		delete pWorkerThread;
-
-		return false;
-	}
-
 	// Insert
+	pair<map<unsigned int, WorkerThread *>::iterator, bool> threadPair;
 	if (write_lock_threads() == true)
 	{
-		m_threads[pWorkerThread] = pThread;
+		threadPair = m_threads.insert(pair<unsigned int, WorkerThread *>(pWorkerThread->getId(), pWorkerThread));
+		if (threadPair.second == false)
+		{
+			delete pWorkerThread;
+			pWorkerThread = NULL;
+		}
 
 		unlock_threads();
 	}
+
+	// Start the thread
+	if (pWorkerThread != NULL)
+	{
+		Thread *pThread = pWorkerThread->start();
+		if (pThread != NULL)
+		{
+			createdThread = true;
+		}
+		else
+		{
+			// Erase
+			if (write_lock_threads() == true)
+			{
+				m_threads.erase(threadPair.first);
+
+				unlock_threads();
+			}
+			delete pWorkerThread;
+		}
+	}
+
 	++m_nextThreadId;
 
-	return true;
+	return createdThread;
 }
 
 unsigned int ThreadsManager::get_threads_count(void)
@@ -546,16 +566,6 @@ unsigned int ThreadsManager::get_threads_count(void)
 	return (unsigned int)max(count , 0);
 }
 
-bool ThreadsManager::has_threads(void)
-{
-	if (m_threads.empty() == true)
-	{
-		return false;
-	}
-
-	return true;
-}
-
 void ThreadsManager::stop_threads(void)
 {
 	if (m_threads.empty() == false)
@@ -564,7 +574,6 @@ void ThreadsManager::stop_threads(void)
 		{
 			// Stop threads
 			for_each(m_threads.begin(), m_threads.end(), StopThreadFunc());
-			m_threads.clear();
 
 			unlock_threads();
 		}
@@ -724,18 +733,6 @@ bool ThreadsManager::pop_queue(const string &urlWasIndexed)
 	return foundItem;
 }
 
-void ThreadsManager::get_statistics(unsigned int &queueSize)
-{
-	if (read_lock_lists() == true)
-	{
-		// We want the number of documents being indexed,
-		// not the number of document waiting in the queue
-		queueSize = m_beingIndexed.size();
-
-		unlock_lists();
-	}
-}
-
 ListerThread::ListerThread(const string &indexName, unsigned int startDoc) :
 	WorkerThread(),
 	m_indexName(indexName),
@@ -1417,7 +1414,6 @@ void DownloadingThread::doWork(void)
 IndexingThread::IndexingThread(const DocumentInfo &docInfo, const string &indexLocation,
 	bool allowAllMIMETypes) :
 	DownloadingThread(docInfo),
-	m_docInfo(docInfo),
 	m_docId(0),
 	m_indexLocation(indexLocation),
 	m_allowAllMIMETypes(allowAllMIMETypes),
@@ -1637,11 +1633,14 @@ void IndexingThread::doWork(void)
 			}
 			else
 			{
-				const set<string> &labels = m_docInfo.getLabels();
 				unsigned int docId = 0;
+#ifdef DEBUG
+				cout << "IndexingThread::doWork: " << m_docInfo.getLabels().size()
+					<< " labels for URL " << m_pDoc->getLocation() << endl;
+#endif
 
 				// Index the document
-				success = wrapFilter.indexDocument(*m_pDoc, labels, docId);
+				success = wrapFilter.indexDocument(*m_pDoc, m_docInfo.getLabels(), docId);
 				if (success == true)
 				{
 					m_docId = docId;
@@ -1938,12 +1937,10 @@ string MonitorThread::getType(void) const
 	return "MonitorThread";
 }
 
-bool MonitorThread::stop(void)
+void MonitorThread::stop(void)
 {
-	m_done = true;
+	WorkerThread::stop();
 	write(m_ctrlWritePipe, "X", 1);
-
-	return true;
 }
 
 void MonitorThread::processEvents(void)
