@@ -188,7 +188,7 @@ mainWindow::mainWindow() :
 	m_pNotebook->set_show_tabs(true);
 	m_pNotebook->set_show_border(true);
 	m_pNotebook->set_tab_pos(Gtk::POS_TOP);
-	m_pNotebook->set_scrollable(false);
+	m_pNotebook->set_scrollable(true);
 	rightVbox->pack_start(*m_pNotebook, Gtk::PACK_EXPAND_WIDGET, 4);
 	m_pageSwitchConnection = m_pNotebook->signal_switch_page().connect(
 		sigc::mem_fun(*this, &mainWindow::on_switch_page), false);
@@ -276,16 +276,8 @@ mainWindow::mainWindow() :
 //
 mainWindow::~mainWindow()
 {
-	// FIXME: delete all "ignored" threads when exiting !!!
-
-	if (m_pSettingsMonitor != NULL)
-	{
-		delete m_pSettingsMonitor;
-	}
-	if (m_pSettingsHandler != NULL)
-	{
-		delete m_pSettingsHandler;
-	}
+	// Don't delete m_pSettingsMonitor and m_pSettingsHandler, threads may need them
+	// Since mainWindow is destroyed when the program exits, it's a leak we can live with
 
 	// Save engines
 	m_pEnginesTree->save();
@@ -853,9 +845,11 @@ void mainWindow::on_index_changed(ustring indexName)
 		if (m_state.write_lock_lists() == true)
 		{
 			int pageNum = m_pNotebook->append_page(*pIndexPage, *pTab);
-			m_pNotebook->pages().back().set_tab_label_packing(false, true, Gtk::PACK_START);
 			if (pageNum >= 0)
 			{
+				m_pNotebook->pages().back().set_tab_label_packing(false, true, Gtk::PACK_START);
+				// Allow reordering
+				m_pNotebook->set_tab_reorderable(*pIndexPage);
 				foundPage = true;
 			}
 
@@ -1128,6 +1122,15 @@ void mainWindow::on_thread_end(WorkerThread *pThread)
 		{
 			m_state.m_browsingIndex = false;
 		}
+		else if (type == "IndexingThread")
+		{
+			IndexingThread *pIndexThread = dynamic_cast<IndexingThread *>(pThread);
+			if (pIndexThread != NULL)
+			{
+				// Get the URL we failed to index
+				indexedUrl = pIndexThread->getURL();
+			}
+		}
 	}
 	// Based on type, take the appropriate action...
 	else if (type == "ListerThread")
@@ -1188,7 +1191,7 @@ void mainWindow::on_thread_end(WorkerThread *pThread)
 	}
 	else if (type == "QueryingThread")
 	{
-		std::map<string, TreeModel::iterator> updatedGroups;
+		set<DocumentInfo> docsToIndex;
 		int pageNum = -1;
 
 		QueryingThread *pQueryThread = dynamic_cast<QueryingThread *>(pThread);
@@ -1198,13 +1201,21 @@ void mainWindow::on_thread_end(WorkerThread *pThread)
 			return;
 		}
 
-		bool wasCorrected;
+		bool wasCorrected = false;
 		QueryProperties queryProps(pQueryThread->getQuery(wasCorrected));
 		ustring queryName(queryProps.getName());
+		string labelName(queryProps.getLabelName());
 		ustring engineName(to_utf8(pQueryThread->getEngineName()));
 		string resultsCharset(pQueryThread->getCharset());
+		set<string> labels;
 		const vector<DocumentInfo> &resultsList = pQueryThread->getDocuments();
 
+		// These are the labels that need to be applied to results
+		if (labelName.empty() == false)
+		{
+			labels.insert(labelName);
+		}
+
 		status = _("Query");
 		status += " ";
 		status += queryName;
@@ -1216,6 +1227,78 @@ void mainWindow::on_thread_end(WorkerThread *pThread)
 		status += _("ended");
 		set_status(status);
 
+		// Suggest the correction to the user
+		if (wasCorrected == true)
+		{
+			ustring correctedQueryName(_("Corrected"));
+
+			if (queryName.empty() == true)
+			{
+				correctedQueryName += "...";
+			}
+			else
+			{
+				correctedQueryName += " ";
+				correctedQueryName += queryName;
+			}
+			queryProps.setName(correctedQueryName);
+			queryProps.setModified(true);
+
+			add_query(queryProps, true);
+		}
+
+		// Index results ?
+		QueryProperties::IndexWhat indexResults = queryProps.getIndexResults();
+		if ((indexResults != QueryProperties::NOTHING) &&
+			(resultsList.empty() == false))
+		{
+			QueryHistory queryHistory(m_settings.getHistoryDatabaseName());
+			set<unsigned int> docsIds, daemonIds;
+
+			for (vector<DocumentInfo>::const_iterator resultIter = resultsList.begin();
+				resultIter != resultsList.end(); ++resultIter)
+			{
+				unsigned int indexId = 0;
+				unsigned int docId = resultIter->getIsIndexed(indexId);
+				float oldestScore = 0;
+				bool isNewResult = false;
+
+				// Is this a new result or a new query's result ?
+				if (queryHistory.hasItem(queryName, engineName, resultIter->getLocation(), oldestScore) <= 0)
+				{
+#ifdef DEBUG
+					cout << "mainWindow::on_thread_end: new result " << resultIter->getLocation() << endl;
+#endif
+					isNewResult = true;
+				}
+				
+				if ((docId == 0) &&
+					((indexResults == QueryProperties::ALL_RESULTS) || (isNewResult == true)))
+				{
+					// This document is not in either index
+					docsToIndex.insert(*resultIter);
+				}
+				else if (labels.empty() == false)
+				{
+					if (indexId == m_settings.getIndexIdByName(_("My Web Pages")))
+					{
+						docsIds.insert(docId);
+					}
+					else if (indexId == m_settings.getIndexIdByName(_("My Documents")))
+					{
+						daemonIds.insert(docId);
+					}
+				}
+			}
+
+			if ((docsIds.empty() == false) ||
+				(daemonIds.empty() == false))
+			{
+				// Apply the new label to existing documents
+				start_thread(new LabelUpdateThread(labels, docsIds, daemonIds, false));
+			}
+		}
+
 		// Find the page for this query
 		ResultsTree *pResultsTree = NULL;
 		ResultsPage *pResultsPage = dynamic_cast<ResultsPage*>(get_page(queryName, NotebookPageBox::RESULTS_PAGE));
@@ -1254,9 +1337,14 @@ void mainWindow::on_thread_end(WorkerThread *pThread)
 			if (m_state.write_lock_lists() == true)
 			{
 				pageNum = m_pNotebook->append_page(*pResultsPage, *pTab);
-				m_pNotebook->pages().back().set_tab_label_packing(false, true, Gtk::PACK_START);
-				// Switch to this new page
-				m_pNotebook->set_current_page(pageNum);
+				if (pageNum >= 0)
+				{
+					m_pNotebook->pages().back().set_tab_label_packing(false, true, Gtk::PACK_START);
+					// Allow reordering
+					m_pNotebook->set_tab_reorderable(*pResultsPage);
+					// Switch to this new page
+					m_pNotebook->set_current_page(pageNum);
+				}
 
 				m_state.unlock_lists();
 			}
@@ -1271,86 +1359,18 @@ void mainWindow::on_thread_end(WorkerThread *pThread)
 				resultsCharset, pQueryThread->isLive());
 		}
 
-		// Suggest the correction to the user
-		if (wasCorrected == true)
+		// Now that results are displayed, go ahead and index documents
+		for (set<DocumentInfo>::const_iterator docIter = docsToIndex.begin();
+			docIter != docsToIndex.end(); ++docIter)
 		{
-			ustring correctedQueryName(_("Corrected"));
-
-			if (queryName.empty() == true)
-			{
-				correctedQueryName += "...";
-			}
-			else
-			{
-				correctedQueryName += " ";
-				correctedQueryName += queryName;
-			}
-			queryProps.setName(correctedQueryName);
-			queryProps.setModified(true);
-
-			add_query(queryProps, true);
-		}
-
-		// Index results ?
-		if ((queryProps.getIndexResults() == true) &&
-			(resultsList.empty() == false))
-		{
-			set<string> labels;
-			set<unsigned int> docsIds, daemonIds;
-			string labelName(queryProps.getLabelName());
-
-			// Set the list of labels
-			if (labelName.empty() == false)
-			{
-				labels.insert(labelName);
-			}
+			DocumentInfo docInfo(*docIter);
 
+			// Set/reset labels
+			docInfo.setLabels(labels);
 #ifdef DEBUG
-			cout << "mainWindow::on_thread_end: indexing results, with label " << labelName << endl;
+			cout << "mainWindow::on_thread_end: indexing results with label " << labelName << endl;
 #endif
-			for (vector<DocumentInfo>::const_iterator resultIter = resultsList.begin();
-				resultIter != resultsList.end(); ++resultIter)
-			{
-				unsigned int indexId;
-				unsigned int docId = resultIter->getIsIndexed(indexId);
-
-				if (docId == 0)
-				{
-					DocumentInfo docInfo(resultIter->getTitle(), resultIter->getLocation(),
-						resultIter->getType(), resultIter->getLanguage());
-
-					// Set/reset labels
-					docInfo.setLabels(labels);
-
-					// This document is not in either index
-					m_state.queue_index(docInfo);
-				}
-				else if (indexId == m_settings.getIndexIdByName(_("My Web Pages")))
-				{
-					if (labelName.empty() == false)
-					{
-						docsIds.insert(docId);
-					}
-				}
-				else if (indexId == m_settings.getIndexIdByName(_("My Documents")))
-				{
-					if (labelName.empty() == false)
-					{
-						daemonIds.insert(docId);
-					}
-				}
-			}
-
-			if ((labelName.empty() == false) &&
-				((docsIds.empty() == false) ||
-				(daemonIds.empty() == false)))
-			{
-				set<string> labels;
-
-				// Apply the new label to existing documents
-				labels.insert(labelName);
-				start_thread(new LabelUpdateThread(labels, docsIds, daemonIds, false));
-			}
+			m_state.queue_index(docInfo);
 		}
 	}
 	else if (type == "ExpandQueryThread")
@@ -1517,19 +1537,13 @@ void mainWindow::on_thread_end(WorkerThread *pThread)
 					(rowsCount < m_maxDocsCount))
 				{
 					// Add a row to the index tree
-					DocumentInfo indexedDoc(docInfo.getTitle(),
-						docInfo.getLocation(), docInfo.getType(),
-						docInfo.getLanguage());
-					indexedDoc.setTimestamp(docInfo.getTimestamp());
-					indexedDoc.setSize(docInfo.getSize());
-
-					append_document(pIndexPage, _("My Web Pages"), indexedDoc);
+					append_document(pIndexPage, _("My Web Pages"), docInfo);
 				}
 				pIndexPage->setDocumentsCount(pIndexPage->getDocumentsCount() + 1);
 				pIndexPage->updateButtonsState(m_maxDocsCount);
 			}
 
-			// FIXME: update the result's indexed status in the results list !
+			// FIXME: update the result's indexed status in results lists !
 		}
 
 		set_status(status);
@@ -2141,12 +2155,18 @@ void mainWindow::on_import_activate()
 	importDialog importBox;
 
 	importBox.show();
-	importBox.run();
+	if (importBox.run() != RESPONSE_OK)
+	{
+		return;
+	}
 
 	// Anything to import ?
 	const DocumentInfo &docInfo = importBox.getDocumentInfo();
 	if (docInfo.getLocation().empty() == false)
 	{
+#ifdef DEBUG
+		cout << "mainWindow::on_import_activate: URL is " << docInfo.getLocation() << endl;
+#endif
 		ustring status = m_state.queue_index(docInfo);
 		if (status.empty() == false)
 		{
@@ -2216,10 +2236,7 @@ void mainWindow::on_refreshindex_activate()
 #endif
 
 		// Add this action to the queue
-		DocumentInfo docInfo(docIter->getTitle(), docIter->getLocation(), 
-			docIter->getType(), docIter->getLanguage());
-		docInfo.setTimestamp(docIter->getTimestamp());
-		docInfo.setSize(docIter->getSize());
+		DocumentInfo docInfo(*docIter);
 
 		ustring status = m_state.queue_index(docInfo);
 		if (status.empty() == false)
@@ -3071,6 +3088,9 @@ void mainWindow::edit_query(QueryProperties &queryProps, bool newQuery)
 			return;
 		}
 
+		// User edits reset the automatically modified flag
+		queryProps.setModified(false);
+
 		set_status(_("Edited query"));
 	}
 	else
@@ -3316,13 +3336,13 @@ void mainWindow::browse_index(const ustring &indexName, const ustring &queryName
 //
 // View documents
 //
-void mainWindow::view_documents(vector<DocumentInfo> &documentsList)
+void mainWindow::view_documents(const vector<DocumentInfo> &documentsList)
 {
 	ViewHistory viewHistory(m_settings.getHistoryDatabaseName());
 	RefPtr<RecentManager> recentManager = RecentManager::get_default();
 	multimap<string, string> locationsByType;
 
-	for (vector<DocumentInfo>::iterator docIter = documentsList.begin();
+	for (vector<DocumentInfo>::const_iterator docIter = documentsList.begin();
 		docIter != documentsList.end(); ++docIter)
 	{
 		string url(docIter->getLocation());
@@ -3477,7 +3497,7 @@ void mainWindow::view_documents(vector<DocumentInfo> &documentsList)
 
 			// ...as well as in the recently used files list
 			RecentManager::Data itemData;
-			for (vector<DocumentInfo>::iterator docIter = documentsList.begin();
+			for (vector<DocumentInfo>::const_iterator docIter = documentsList.begin();
 				docIter != documentsList.end(); ++docIter)
 			{
 				if (docIter->getLocation() == url)
