@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2006 Fabrice Colin
+ *  Copyright 2005-2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,23 +17,24 @@
  */
 
 #include <stdlib.h>
-#include <signal.h>
 #include <libintl.h>
 #include <getopt.h>
 #include <sys/time.h>
 #include <sys/resource.h>
 #include <sys/types.h>
+#include <signal.h>
 #include <unistd.h>
+#include <errno.h>
 #include <iostream>
+#include <sstream>
 #include <fstream>
+#include <cstring>
 #include <sigc++/sigc++.h>
 #include <glibmm.h>
 #include <glibmm/thread.h>
 #include <glibmm/ustring.h>
 #include <glibmm/miscutils.h>
 #include <glibmm/convert.h>
-#include <glibmm/object.h>
-#include <glibmm/main.h>
 
 #include "config.h"
 #include "NLS.h"
@@ -57,8 +58,9 @@ static string g_pidFileName;
 static streambuf *g_coutBuf = NULL;
 static streambuf *g_cerrBuf = NULL;
 static struct option g_longOptions[] = {
-	{"fullscan", 1, 0, 'f'},
+	{"fullscan", 0, 0, 'f'},
 	{"help", 0, 0, 'h'},
+	{"ignore-version", 0, 0, 'i'},
 	{"priority", 1, 0, 'p'},
 	{"reindex", 0, 0, 'r'},
 	{"version", 0, 0, 'v'},
@@ -115,6 +117,8 @@ static void quitAll(int sigNum)
 
 static DBusHandlerResult filterHandler(DBusConnection *pConnection, DBusMessage *pMessage, void *pData)
 {
+	DaemonState *pServer = (DaemonState *)pData;
+
 #ifdef DEBUG
 	cout << "filterHandler: called" << endl;
 #endif
@@ -124,6 +128,10 @@ static DBusHandlerResult filterHandler(DBusConnection *pConnection, DBusMessage
 #ifdef DEBUG
 		cout << "filterHandler: received Disconnected" << endl;
 #endif
+		if (pServer != NULL)
+		{
+			pServer->set_flag(DaemonState::DISCONNECTED);
+		}
 		quitAll(0);
 	}
 	else if (dbus_message_is_signal(pMessage, DBUS_INTERFACE_DBUS, "NameOwnerChanged") == TRUE)
@@ -149,8 +157,6 @@ static DBusHandlerResult filterHandler(DBusConnection *pConnection, DBusMessage
 			DBUS_TYPE_INVALID) == TRUE) &&
 			(pData != NULL))
 		{
-			DaemonState *pServer = (DaemonState *)pData;
-
 			if (dbus_message_is_signal(pMessage, "org.gnome.PowerManager", "OnAcChanged") == TRUE)
 			{
 				// This tells us if we are on AC, not on battery
@@ -167,16 +173,22 @@ static DBusHandlerResult filterHandler(DBusConnection *pConnection, DBusMessage
 			if (onBattery == TRUE)
 			{
 				// We are now on battery
-				pServer->set_flag(DaemonState::ON_BATTERY);
-				pServer->stop_crawling();
+				if (pServer != NULL)
+				{
+					pServer->set_flag(DaemonState::ON_BATTERY);
+					pServer->stop_crawling();
+				}
 
 				cout << "System is now on battery" << endl;
 			}
 			else
 			{
 				// Back on-line
-				pServer->reset_flag(DaemonState::ON_BATTERY);
-				pServer->start_crawling();
+				if (pServer != NULL)
+				{
+					pServer->reset_flag(DaemonState::ON_BATTERY);
+					pServer->start_crawling();
+				}
 
 				cout << "System is now on AC" << endl;
 			}
@@ -196,12 +208,7 @@ static void unregisteredHandler(DBusConnection *pConnection, void *pData)
 
 static DBusHandlerResult messageHandler(DBusConnection *pConnection, DBusMessage *pMessage, void *pData)
 {
-	DaemonState *pServer = NULL;
-
-	if (pData != NULL)
-	{
-		pServer = (DaemonState *)pData;
-	}
+	DaemonState *pServer = (DaemonState *)pData;
 
 	if ((pConnection != NULL) &&
 		(pMessage != NULL))
@@ -209,7 +216,10 @@ static DBusHandlerResult messageHandler(DBusConnection *pConnection, DBusMessage
 		dbus_connection_ref(pConnection);
 		dbus_message_ref(pMessage);
 
-		pServer->start_thread(new DBusServletThread(pServer, pConnection, pMessage));
+		if (pServer != NULL)
+		{
+			pServer->start_thread(new DBusServletThread(pServer, pConnection, pMessage));
+		}
 	}
 
 	return DBUS_HANDLER_RESULT_HANDLED;
@@ -264,9 +274,10 @@ int main(int argc, char **argv)
 	bool resetLabels = false;
 	bool fullScan = false;
 	bool reindex = false;
+	bool ignoreVersion = false;
 
 	// Look at the options
-	int optionChar = getopt_long(argc, argv, "fhp:rv", g_longOptions, &longOptionIndex);
+	int optionChar = getopt_long(argc, argv, "fhip:rv", g_longOptions, &longOptionIndex);
 	while (optionChar != -1)
 	{
 		switch (optionChar)
@@ -281,11 +292,15 @@ int main(int argc, char **argv)
 					<< "Options:\n"
 					<< "  -f, --fullscan	force a full scan\n"
 					<< "  -h, --help		display this help and exit\n"
+					<< "  -i, --ignore-version	ignore the index version number\n"
 					<< "  -p, --priority	set the daemon's priority (default 15)\n"
 					<< "  -r, --reindex		force a reindex\n"
 					<< "  -v, --version		output version information and exit\n"
 					<< "\nReport bugs to " << PACKAGE_BUGREPORT << endl;
 				return EXIT_SUCCESS;
+			case 'i':
+				ignoreVersion = true;
+				break;
 			case 'p':
 				if (optarg != NULL)
 				{
@@ -311,7 +326,7 @@ int main(int argc, char **argv)
 		}
 
 		// Next option
-		optionChar = getopt_long(argc, argv, "fhp:rv", g_longOptions, &longOptionIndex);
+		optionChar = getopt_long(argc, argv, "fhip:rv", g_longOptions, &longOptionIndex);
 	}
 
 #if defined(ENABLE_NLS)
@@ -338,6 +353,8 @@ int main(int argc, char **argv)
 	// This should make Xapian use Flint rather than Quartz
 	Glib::setenv("XAPIAN_PREFER_FLINT", "1");
 
+	// Make the locale follow the environment variables
+	setlocale(LC_ALL, "");
 	char *pLocale = setlocale(LC_ALL, NULL);
 	if (pLocale != NULL)
 	{
@@ -345,19 +362,28 @@ int main(int argc, char **argv)
 
 		if (locale != "C")
 		{
+			bool appendUTF8 = false;
+
 			string::size_type pos = locale.find_last_of(".");
-			if (pos != string::npos)
+			if ((pos != string::npos) &&
+				((strcasecmp(locale.substr(pos).c_str(), ".utf8") != 0) &&
+				(strcasecmp(locale.substr(pos).c_str(), ".utf-8") != 0)))
 			{
 				locale.resize(pos);
+				appendUTF8 = true;
 			}
-			locale += ".UTF-8";
 
-			pLocale = setlocale(LC_ALL, locale.c_str());
-			if (pLocale != NULL)
+			if (appendUTF8 == true)
 			{
+				locale += ".UTF-8";
+
+				pLocale = setlocale(LC_ALL, locale.c_str());
+				if (pLocale != NULL)
+				{
 #ifdef DEBUG
-				cout << "Changed locale to " << pLocale << endl;
+					cout << "Changed locale to " << pLocale << endl;
 #endif
+				}
 			}
 		}
 	}
@@ -370,7 +396,7 @@ int main(int argc, char **argv)
 	string confDirectory = PinotSettings::getConfigurationDirectory();
 	if (chdir(confDirectory.c_str()) == 0)
 	{
-		ofstream pidFile;
+		fstream pidFile;
 		string fileName(confDirectory);
 
 		// Redirect cout and cerr to a file
@@ -381,11 +407,70 @@ int main(int argc, char **argv)
 		cout.rdbuf(g_outputFile.rdbuf());
 		cerr.rdbuf(g_outputFile.rdbuf());
 
-		// Save the PID to file
+		// Open the PID file
 		g_pidFileName = confDirectory + "/pinot-dbus-daemon.pid";
-		pidFile.open(g_pidFileName.c_str());
-		pidFile << getpid() << endl;
-		pidFile.close();
+		pidFile.open(g_pidFileName.c_str(), std::ios::in);
+		if (pidFile.is_open() == true)
+		{
+			pid_t daemonPID = 0;
+			bool checkProcess = true, daemonDied = false;
+
+			pidFile >> daemonPID;
+			pidFile.close();
+
+			// Is another daemon running ?
+			if (daemonPID > 0)
+			{
+				fstream cmdLineFile;
+				stringstream cmdLineFileName;
+
+				// FIXME: check for existence of /proc
+				cmdLineFileName << "/proc/" << daemonPID << "/cmdline";
+				cmdLineFile.open(cmdLineFileName.str().c_str(), std::ios::in);
+				if (cmdLineFile.is_open() == true)
+				{
+					string cmdLine;
+
+					cmdLineFile >> cmdLine;
+					cmdLineFile.close();
+
+					if (cmdLine.find("pinot-dbus-daemon") == string::npos)
+					{
+						// It's another process
+						checkProcess = false;
+						daemonDied = true;
+					}
+				}
+
+				if (checkProcess == true)
+				{
+					if (kill(daemonPID, 0) == 0)
+					{
+						// It's still running
+						cout << "Daemon instance " << daemonPID << " is still running" << endl;
+						return EXIT_SUCCESS;
+					}
+					else if (errno == ESRCH)
+					{
+						// This PID doesn't exist
+						daemonDied = true;
+					}
+				}
+
+				if (daemonDied == true)
+				{
+					cerr << "Previous daemon instance " << daemonPID << " died prematurely" << endl;
+				}
+			}
+		}
+
+		// Now save our PID
+		pidFile.open(g_pidFileName.c_str(), std::ios::out);
+		if (pidFile.is_open() == true)
+		{
+			pidFile << getpid() << endl;
+			pidFile.close();
+		}
 	}
 
 	// Initialize utility classes
@@ -430,7 +515,6 @@ int main(int argc, char **argv)
 	newAction.sa_flags = 0;
 	newAction.sa_handler = quitAll;
 	sigaction(SIGINT, &newAction, NULL);
-	sigaction(SIGKILL, &newAction, NULL);
 	sigaction(SIGQUIT, &newAction, NULL);
 
 	// Open the daemon index in read-write mode 
@@ -507,13 +591,14 @@ int main(int argc, char **argv)
 
 	DBusError error;
 	DaemonState server;
+	IndexInterface *pIndex = NULL;
 
 	dbus_error_init(&error);
 	dbus_connection_set_exit_on_disconnect(pConnection, FALSE);
 	dbus_connection_setup_with_g_main(pConnection, NULL);
 
 	if (dbus_connection_register_object_path(pConnection, g_pinotDBusObjectPath,
-			&g_callVTable, &server) == TRUE)
+		&g_callVTable, &server) == TRUE)
 	{
 		// Request to be identified by this name
 		// FIXME: flags are currently broken ?
@@ -544,13 +629,20 @@ int main(int argc, char **argv)
 			bool gotLabels = false;
 			bool onBattery = false;
 
-			IndexInterface *pIndex = settings.getIndex(settings.m_daemonIndexLocation);
+			pIndex = settings.getIndex(settings.m_daemonIndexLocation);
 			if (pIndex != NULL)
 			{
-				string indexVersion(pIndex->getVersion());
+				string indexVersion(pIndex->getMetadata("version"));
+
 				gotLabels = pIndex->getLabels(labels);
+
 				// What version is the index at ?
-				if (indexVersion < PINOT_INDEX_MIN_VERSION)
+				if (indexVersion.empty() == true)
+				{
+					indexVersion = "0.0";
+				}
+				if ((ignoreVersion == false) &&
+					(indexVersion < PINOT_INDEX_MIN_VERSION))
 				{
 					cout << "Upgrading index from version " << indexVersion << " to " << VERSION << endl;
 
@@ -565,7 +657,9 @@ int main(int argc, char **argv)
 
 					resetHistory = resetLabels = true;
 				}
-				pIndex->setVersion(VERSION);
+
+				pIndex->setMetadata("version", VERSION);
+				pIndex->setMetadata("dbus-status", "Running");
 			}
 
 			if (resetHistory == true)
@@ -603,11 +697,6 @@ int main(int argc, char **argv)
 				}
 			}
 
-			if (pIndex != NULL)
-			{
-				delete pIndex;
-			}
-
 			// Connect to the quit signal
 			server.getQuitSignal().connect(sigc::ptr_fun(&quitAll));
 
@@ -671,6 +760,23 @@ int main(int argc, char **argv)
 	}
 	dbus_error_free(&error);
 
+	if (pIndex != NULL)
+	{
+		if (server.is_flag_set(DaemonState::DISCONNECTED) == true)
+		{
+			pIndex->setMetadata("dbus-status", "Disconnected");
+		}
+		else if (server.is_flag_set(DaemonState::STOPPED) == true)
+		{
+			pIndex->setMetadata("dbus-status", "Stopped");
+		}
+		else
+		{
+			pIndex->setMetadata("dbus-status", "");
+		}
+		delete pIndex;
+	}
+
 	// Stop everything
 	server.disconnect();
 	server.stop_threads();
