@@ -24,12 +24,6 @@
 #else
 #include <unistd.h>
 #endif
-#ifdef HAVE_ARPA_INET_H
-# include <arpa/inet.h>
-#endif
-#ifdef HAVE_NETINET_IN_H
-# include <netinet/in.h>
-#endif
 #include <ctype.h>
 #include <algorithm>
 
@@ -115,62 +109,6 @@ string StringManip::extractField(const string &str, const string &start, const s
 	return fieldValue;
 }
 
-/// Replaces entities.
-string StringManip::replaceEntities(const string &str)
-{
-	// FIXME: replace all
-	static const char *escapedChars[] = { "quot", "amp", "lt", "gt", "nbsp", "eacute", "egrave", "agrave", "ccedil"};
-	static const char *unescapedChars[] = { "\"", "&", "<", ">", " ", "e", "e", "a", "c"};
-	static const unsigned int escapedCharsCount = 9;
-	string unescapedStr;
-	string::size_type startPos = 0;
-
-	string::size_type pos = str.find("&");
-	while (pos != string::npos)
-	{
-		unescapedStr += str.substr(startPos, pos - startPos);
-
-		startPos = pos + 1;
-		pos = str.find(";", startPos);
-		if ((pos != string::npos) &&
-			(pos < startPos + 10))
-		{
-			string escapedChar(str.substr(startPos, pos - startPos));
-			bool replacedChar = false;
-
-			// See if we can replace this with an actual character
-			for (unsigned int count = 0; count < escapedCharsCount; ++count)
-			{
-				if (escapedChar == escapedChars[count])
-				{
-					unescapedStr += unescapedChars[count];
-					replacedChar = true;
-					break;
-				}
-			}
-
-			if (replacedChar == false)
-			{
-				// This couldn't be replaced, leave it as it is...
-				unescapedStr += "&";
-				unescapedStr += escapedChar;
-				unescapedStr += ";";
-			}
-
-			startPos = pos + 1;
-		}
-
-		// Next
-		pos = str.find("&", startPos);
-	}
-	if (startPos < str.length())
-	{
-		unescapedStr  += str.substr(startPos);
-	}
-
-	return unescapedStr;
-}
-
 /// Replaces a sub-string.
 string StringManip::replaceSubString(const string &str, const std::string &substr, const std::string &rep)
 {
@@ -203,27 +141,10 @@ string StringManip::replaceSubString(const string &str, const std::string &subst
 	return cleanStr;
 }
 
-/// Removes characters from a string.
-unsigned int StringManip::removeCharacters(string &str, const string &characters)
-{
-	unsigned int count = 0;
-
-	string::size_type charPos = str.find_first_of(characters.c_str());
-	while (charPos != string::npos)
-	{
-		str.erase(charPos, 1);
-		++count;
-
-		charPos = str.find_first_of(characters.c_str(), charPos);
-	}
-
-	return count;	
-}
-
 /// Trims spaces at the start and end of a string.
 unsigned int StringManip::trimSpaces(string &str)
 {
-	unsigned int pos = 0;
+	string::size_type pos = 0;
 	unsigned int count = 0;
 
 	while ((str.empty() == false) && (pos < str.length()))
@@ -238,7 +159,7 @@ unsigned int StringManip::trimSpaces(string &str)
 		++count;
 	}
 
-	for (unsigned int pos = str.length() - 1;
+	for (pos = str.length() - 1;
 		(str.empty() == false) && (pos >= 0); --pos)
 	{
 		if (isspace(str[pos]) == 0)
@@ -338,28 +259,3 @@ string StringManip::hashString(const string &str, unsigned int maxLength)
 	return result;
 }
 
-/// Converts a binary string to an integer.
-uint32_t StringManip::binaryStringToInteger(const string &str)
-{
-	uint32_t value;
-
-	// This is from from Xapian's xapian-applications/omega/values.h
-	if (str.size() != 4)
-	{
-		return (uint32_t)-1;
-	}
-
-	memcpy(&value, str.data(), 4);
-
-	return ntohl(value);
-}
-
-/// Converts an integer to a binary string.
-string StringManip::integerToBinaryString(uint32_t value)
-{
-	// This is from from Xapian's xapian-applications/omega/values.h
-	value = htonl(value);
-
-	return string(reinterpret_cast<const char*>(&value), 4);
-}
-
