@@ -24,7 +24,7 @@ using std::string;
 static const int g_rfc2396Encoded[] = {
 	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  /* 0x00 - 0x0f */
 	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  /* 0x10 - 0x1f */
-	1, 0, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /*  ' ' - '/'  */
+	1, 0, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0,  /*  ' ' - '/'  */
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0,  /*  '0' - '?'  */
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /*  '@' - 'O'  */
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0,  /*  'P' - '_'  */
@@ -78,8 +78,8 @@ Url& Url::operator=(const Url& other)
 
 void Url::parse(const string &url)
 {
-	string::size_type pos1 =0, pos2 = 0;
-	bool hasHostName = true;
+	string::size_type pos1 = 0, pos2 = 0;
+	bool hasHostName = true, hasParameters = true;
 
 	if ((url[0] == '/') ||
 		(url[0] == '.'))
@@ -92,7 +92,6 @@ void Url::parse(const string &url)
 
 		// Assume default protocol
 		m_protocol = "file";
-
 		hasHostName = false;
 	}
 	else
@@ -101,7 +100,7 @@ void Url::parse(const string &url)
 		pos1 = url.find("://");
 		if (pos1 != string::npos)
 		{
-			m_protocol = url.substr(0, pos1);
+			m_protocol = StringManip::toLowerCase(url.substr(0, pos1));
 			pos1 += 3;
 		}
 		else
@@ -118,6 +117,11 @@ void Url::parse(const string &url)
 		}
 	}
 
+	if (m_protocol == "file")
+	{
+		hasParameters = false;
+	}
+
 	if (hasHostName == true)
 	{
 		string userAndPassword;
@@ -182,13 +186,16 @@ void Url::parse(const string &url)
 		m_host = "localhost";
 	}
 
-	string locationAndFile = url.substr(pos2);
-	pos2 = locationAndFile.find("?");
+	string locationAndFile(url.substr(pos2));
 	// Parameters
-	if (pos2 != string::npos)
+	if (hasParameters == true)
 	{
-		m_parameters = locationAndFile.substr(pos2+1);
-		locationAndFile.resize(pos2);
+		pos2 = locationAndFile.find("?");
+		if (pos2 != string::npos)
+		{
+			m_parameters = locationAndFile.substr(pos2+1);
+			locationAndFile.resize(pos2);
+		}
 	}
 
 	// Location and file
@@ -357,9 +364,9 @@ string Url::reduceHost(const string &hostName, unsigned int level)
 		}
 		else
 		{
-			string level(hostName.substr(pos + 1, endPos - pos));
-			level += reducedHost;
-			reducedHost = level;
+			string levelled(hostName.substr(pos + 1, endPos - pos));
+			levelled += reducedHost;
+			reducedHost = levelled;
 		}
 
 		// Next
